/**
 * Control for adding/deleting multiple pieces of contact information
 *
 * @module package/quiqqer/locations/bin/controls/attributes/ContactInformation
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/locations/bin/controls/attributes/ContactInformation', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/buttons/Button',
    'qui/controls/buttons/Select',
    'Locale',

    'css!package/quiqqer/locations/bin/controls/attributes/ContactInformation.css'

], function (QUI, QUIControl, QUIButton, QUISelect, QUILocale) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/controls/attributes/ContactInformation',

        Binds: [
            '$onInject',
            '$onImport',
            '$build',
            '$getInfoElm',
            '$setValue'
        ],

        options: {
            contactInfo: [] // multiple contact information entries
        },

        initialize: function (options) {
            this.parent(options);

            this.addEvents({
                onInject: this.$onInject,
                onImport: this.$onImport
            });

            this.$Input       = null;
            this.$contactInfo = this.getAttribute('contactInfo');
            this.$InfoElm     = null;
            this.$initialLoad = true;
        },

        /**
         * create the domnode element
         *
         * @return {HTMLDivElement}
         */
        create: function () {
            var self = this;

            this.$Elm = new Element('div', {
                'class': 'quiqqer-contact-info-parent',
                html   : '<div class="quiqqer-contact-info-list"></div>' +
                '<div class="quiqqer-contact-info-add"></div>'
            });

            new QUIButton({
                textimage: 'fa fa-plus',
                text     : QUILocale.get(lg, 'controls.attributes.contactInformation.add.btn.text'),
                alt      : QUILocale.get(lg, 'controls.attributes.contactInformation.add.btn.text'),
                title    : QUILocale.get(lg, 'controls.attributes.contactInformation.add.btn.text'),
                events   : {
                    onClick: function () {
                        self.$getInfoElm().inject(self.$InfoElm);
                    }
                }
            }).inject(
                this.$Elm.getElement('.quiqqer-contact-info-add')
            );

            this.$InfoElm = this.$Elm.getElement('.quiqqer-contact-info-list');

            return this.$Elm;
        },

        /**
         * event : onInject
         */
        $onInject: function () {
            this.$initialLoad = false;
            this.$build();
        },

        /**
         * event: onImport
         */
        $onImport: function () {
            this.$Input      = this.getElm();
            this.$Input.type = 'hidden';

            if (this.$Input.value !== '') {
                this.$contactInfo = JSON.decode(this.$Input.value);
            }

            var Elm = this.create();
            Elm.inject(this.$Input, 'after');

            if (this.$Input.getParent('.field-container')) {
                var Container = new Element('div', {
                    'class': 'field-container-field'
                });

                Container.wraps(Elm);
            }

            this.$build();
            this.$initialLoad = false;
        },

        /**
         * Build elements
         */
        $build: function () {
            for (var i = 0, len = this.$contactInfo.length; i < len; i++) {
                var Ifno = this.$contactInfo[i];
                var Elm    = this.$getInfoElm().inject(this.$InfoElm);

                var TypeSelect = QUI.Controls.getById(
                    Elm.getElement(
                        '.quiqqer-contact-info-type .qui-select'
                    ).getProperty('data-quiid')
                );

                TypeSelect.setValue(Ifno.type);

                Elm.getElement(
                    '.quiqqer-contact-info-value input'
                ).value = Ifno.value;
            }
        },

        /**
         * Get HTMLElement for one number
         *
         * @return {HTMLElement}
         */
        $getInfoElm: function () {
            var self = this;

            var InfoElm = new Element('div', {
                'class': 'quiqqer-contact-info',
                html   : '<div class="quiqqer-contact-info-type"></div>' +
                '<div class="quiqqer-contact-info-value">' +
                '<input type="tel">' +
                '</div>' +
                '<div class="quiqqer-contact-info-removebtn"></div>'
            });

            // input
            InfoElm.getElement('.quiqqer-contact-info-value input').addEvents({
                keyup: function () {
                    if (!self.$initialLoad) {
                        self.$setValue();
                    }
                }
            });

            // type select
            var TypeSelect = new QUISelect({
                'class': 'quiqqer-contact-info-type-select'
            }).inject(
                InfoElm.getElement('.quiqqer-contact-info-type')
            );

            TypeSelect.appendChild(
                QUILocale.get(lg, 'controls.attributes.contactInformation.type.phone'),
                'phone',
                'fa fa-phone'
            ).appendChild(
                QUILocale.get(lg, 'controls.attributes.contactInformation.type.mobile'),
                'mobile',
                'fa fa-mobile'
            ).appendChild(
                QUILocale.get(lg, 'controls.attributes.contactInformation.type.fax'),
                'fax',
                'fa fa-fax'
            ).appendChild(
                QUILocale.get(lg, 'controls.attributes.contactInformation.type.email'),
                'email',
                'fa fa-envelope'
            ).appendChild(
                QUILocale.get(lg, 'controls.attributes.contactInformation.type.website'),
                'website',
                'fa fa-globe'
            );

            TypeSelect.setValue('phone');
            TypeSelect.addEvents({
                onChange: function () {
                    if (!self.$initialLoad) {
                        self.$setValue();
                    }
                }
            });

            // remove btn
            new QUIButton({
                icon  : 'fa fa-times',
                events: {
                    onClick: function (Btn, event) {
                        event.stop();

                        InfoElm.destroy();
                        self.$setValue();
                    }
                }
            }).inject(
                InfoElm.getElement('.quiqqer-contact-info-removebtn')
            );

            return InfoElm;
        },

        /**
         * Get value
         *
         * @return {Object}
         */
        getValue: function () {
            return this.$contactInfo;
        },

        /**
         * Parse all contact info inputs and set value
         */
        $setValue: function () {
            var infoElements = this.$InfoElm.getElements(
                '.quiqqer-contact-info'
            );

            this.$contactInfo = [];

            for (var i = 0, len = infoElements.length; i < len; i++) {
                var InfoElm = infoElements[i];

                // Value Input
                var ValueInput = InfoElm.getElement(
                    '.quiqqer-contact-info-value input'
                );

                var value = ValueInput.value.trim();

                // ignore empty info
                if (value === '') {
                    continue;
                }

                var TypeSelect = QUI.Controls.getById(
                    InfoElm.getElement(
                        '.quiqqer-contact-info-type .qui-select'
                    ).getProperty('data-quiid')
                );

                this.$contactInfo.push({
                    type : TypeSelect.getValue(),
                    value: value
                });
            }

            if (this.$Input) {
                this.$Input.value = JSON.encode(this.$contactInfo);
            }
        }
    });
});
