/**
 * Control setting (regular) opening hours for a location
 *
 * @module package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHours
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onAddEntry [Entry, this]
 * @event onRemoveEntry [this]
 */
define('package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHours', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/buttons/Button',
    'qui/controls/loader/Loader',

    'Locale',
    'package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHoursEntry',

    'css!package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHours.css'

], function (QUI, QUIControl, QUIButton, QUILoader, QUILocale, OpeningHoursEntry) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/controls/attributes/OpeningHours/OpeningHours',

        Binds: [
            '$onInject',
            '$onImport',
            '$setValue',
            'setValue',
            '$setEntries',
            'getValue'
        ],

        options: {
            showAddBtn: true    // show button to manually add a time entry
        },

        initialize: function (options) {
            this.parent(options);

            this.addEvents({
                //onInject: this.$onInject,
                onImport: this.$onImport
            });

            this.Loader = new QUILoader();

            this.$Input     = null;
            this.$EntryList = null;
            this.$value     = null;
            this.$Entries   = {};
        },

        /**
         * create the domnode element
         *
         * @return {HTMLDivElement}
         */
        create: function () {
            var self = this;

            this.$Elm = new Element('div', {
                'class': 'quiqqer-locations-openinghours',
                html   : '<div class="quiqqer-locations-openinghours-list"></div>' +
                    '<div class="quiqqer-locations-openinghours-add"></div>'
            });

            this.Loader.inject(this.$Elm);
            this.Loader.show();

            this.$EntryList = this.$Elm.getElement('.quiqqer-locations-openinghours-list');

            if (this.getAttribute('showAddBtn')) {
                new QUIButton({
                    textimage: 'fa fa-plus',
                    text     : QUILocale.get(lg, 'controls.attributes.openinghours.btn.add.text'),
                    title    : QUILocale.get(lg, 'controls.attributes.openinghours.btn.add.text'),
                    alt      : QUILocale.get(lg, 'controls.attributes.openinghours.btn.add.text'),
                    events   : {
                        onClick: function () {
                            self.$addEntry();
                        }
                    }
                }).inject(
                    this.$Elm.getElement('.quiqqer-locations-openinghours-add')
                );
            }

            this.Loader.hide();

            return this.$Elm;
        },

        /**
         * Add single time entry
         *
         * @return package/quiqqer/locations/bin/controls/attributes/OpeningHoursEntry
         */
        $addEntry: function () {
            var self = this;

            var EntryElm = new Element('div', {
                'class': 'quiqqer-locations-openinghours-list-entry',
                html   : '<div class="quiqqer-locations-openinghours-list-entry-time"></div>' +
                    '<div class="quiqqer-locations-openinghours-list-entry-remove"></div>'
            }).inject(this.$EntryList);

            var Entry = new OpeningHoursEntry({
                events: {
                    onChange: self.$setValue
                }
            }).inject(
                EntryElm.getElement('.quiqqer-locations-openinghours-list-entry-time')
            );

            var entryNo = 'entry-' + Object.getLength(this.$Entries);

            this.$Entries[entryNo] = Entry;

            if (Object.getLength(this.$Entries) <= 1) {
                return Entry;
            }

            // remove btn
            new Element('span', {
                'class': 'fa fa-close',
                events : {
                    click: function (event) {
                        event.stop();

                        EntryElm.destroy();
                        delete self.$Entries[entryNo];
                        self.$setValue();

                        self.fireEvent('removeEntry', [self]);
                    }
                }
            }).inject(
                EntryElm.getElement('.quiqqer-locations-openinghours-list-entry-remove')
            );

            this.fireEvent('addEntry', [Entry, this]);

            return Entry;
        },

        /**
         * Parses all time inputs and sets value
         */
        $setValue: function () {
            var entryValues = [];

            for (var entry in this.$Entries) {
                if (!this.$Entries.hasOwnProperty(entry)) {
                    continue;
                }

                var Entry = this.$Entries[entry];
                entryValues.push(Entry.getValue());
            }

            this.$value = entryValues;

            if (this.$Input) {
                this.$Input.value = JSON.encode(entryValues);
            }

            this.fireEvent('change', [this]);
        },

        /**
         * Set entries from input value
         *
         * @param {array} entries
         */
        setEntries: function (entries) {
            for (var i = 0, len = entries.length; i < len; i++) {
                var NewEntry = this.$addEntry();
                NewEntry.setValue(entries[i]);
            }

            this.$setValue();
        },

        /**
         * Get current value
         *
         * @return {Object}
         */
        getValue: function () {
            return this.$value;
        },

        /**
         * Delete all opening hour entries
         */
        clear: function () {
            this.$Elm.getElements('.quiqqer-locations-openinghours-list-entry').destroy();
            this.$Entries = {};

            this.$setValue();
        },

        /**
         * event: onImport
         */
        $onImport: function () {
            this.$Input      = this.getElm();
            this.$Input.type = 'hidden';

            var Elm = this.create();
            Elm.inject(this.$Input, 'after');

            if (this.$Input.getParent('.field-container')) {
                var Container = new Element('div', {
                    'class': 'field-container-field'
                });

                Container.wraps(Elm);
            }

            if (this.$Input.value !== '') {
                this.setEntries(JSON.decode(this.$Input.value));
            } else {
                this.$addEntry();
            }
        }
    });
});
