/**
 * Display and edit data of a single location
 *
 * @author www.pcsg.de (Henning Leutz)
 * @authro www.pcsg.de (Patrick Müller)
 * @module package/quiqqer/locations/bin/controls/locations/Location
 *
 * @event onSave [this] - If the user saves the location data
 */
define('package/quiqqer/locations/bin/controls/locations/Location', [

    'qui/QUI',
    'qui/controls/desktop/Panel',
    'qui/controls/buttons/Button',
    'qui/controls/buttons/ButtonSwitch',
    'qui/controls/buttons/Separator',
    'qui/utils/Object',

    'Ajax',
    'Locale',
    'Mustache',

    'utils/Controls',
    'utils/Lock',

    'package/quiqqer/locations/bin/Locations',

    'text!package/quiqqer/locations/bin/controls/locations/LocationInfo.html',
    'css!package/quiqqer/locations/bin/controls/locations/Location.css',
    'css!controls/desktop/panels/XML.css'

], function (QUI, QUIPanel, QUIButton, QUIButtonSwitch, QUISeparator,
             QUIObjectUtils, QUIAjax, QUILocale, Mustache, ControlUtils,
             QUILocker, Locations, infoTemplate) {
    "use strict";

    var lg = 'quiqqer/locations';

    /**
     * @class package/quiqqer/locations/bin/controls/locations/Location
     *
     * @param {number} locationId - ID of Location
     * @param {string} [icon] - Panel icon
     */
    return new Class({

        Extends: QUIPanel,
        Type   : 'package/quiqqer/locations/bin/controls/locations/Location',

        Binds: [
            '$onCreate',
            '$onDestroy',
            '$locationRefresh',
            '$onCategoryActive',
            'loadInfoCategory',
            'loadCategory',
            'unloadCategory',
            'save',
            '$checkLock',
            '$showLockInfo',
            '$onActivationStatusChange'
        ],

        options: {
            locationId: false,
            icon      : 'fa fa-home'
        },

        initialize: function (options) {
            this.parent(options);

            this.$Control             = null;
            this.$Location            = null;
            this.$lockKey             = false;
            this.$initialLoad         = true;
            this.$isLocked            = true;
            this.$locationRefreshMode = false;
            this.$CurrentCategory     = null;
            this.$ActiveSwitch        = null;

            this.addEvents({
                onCreate : this.$onCreate,
                onDestroy: this.$onDestroy,
                onRefresh: this.$locationRefresh
            });
        },

        /**
         * Internal creation
         */
        $onCreate: function () {
            var self = this;

            this.getCategoryBar().clear();

            this.addCategory(new QUIButton({
                textimage: 'fa fa-info',
                text     : 'Information',
                events   : {
                    onActive: self.loadInfoCategory
                }
            }));

            this.Loader.show();

            QUIAjax.get([
                'package_quiqqer_locations_ajax_attributes_getCategories'
            ], function (categories) {
                self.getButtonBar().clear();

                // load categories
                for (var i = 0, len = categories.length; i < len; i++) {
                    var Category = new QUIButton(categories[i]);

                    Category.addEvents({
                        onActive: self.$onCategoryActive
                    });

                    self.addCategory(Category);
                }

                // load buttons
                self.addButton({
                    name     : 'save',
                    text     : QUILocale.get('quiqqer/system', 'desktop.panels.xml.btn.save'),
                    textimage: 'fa fa-save',
                    events   : {
                        onClick: self.save
                    }
                });

                self.addButton({
                    name     : 'reload',
                    text     : QUILocale.get('quiqqer/system', 'desktop.panels.xml.btn.cancel'),
                    textimage: 'fa fa-ban',
                    events   : {
                        onClick: function () {
                            self.$locationRefreshMode = true;
                            self.$onCreate();
                        }
                    }
                });

                self.addButton(new QUISeparator());

                self.$ActiveSwitch = new QUIButtonSwitch({
                    name  : 'status',
                    text  : QUILocale.get(lg, 'controls.locations.location.active.off'),
                    status: 0,
                    events: {
                        onChange: self.$onActivationStatusChange
                    }
                });

                self.addButton(self.$ActiveSwitch);

                self.$Elm.addClass('quiqqer-locations-location-panel');

                self.$lockKey = 'location_' + Location.id;
                self.$checkLock();

                self.fireEvent('createEnd', [self]);

                self.$locationRefresh().then(function () {
                    self.$initialLoad = false;
                    self.getCategoryBar().firstChild().click();
                });
            }, {
                'package' : 'quiqqer/locations',
                locationId: this.getAttribute('locationId')
            });
        },

        /**
         * Refresh location data
         *
         * @return {Promise}
         */
        $locationRefresh: function () {
            var self = this;

            this.Loader.show();

            this.$ActiveSwitch.disable();

            return Locations.getLocation(this.getAttribute('locationId')).then(function (Location) {
                self.$Location = Location;

                var name = JSON.decode(self.$Location.attributes.name);

                self.setAttribute(
                    'title',
                    QUILocale.get(lg, 'controls.locations.location.title', {
                        locationId  : self.$Location.id,
                        locationName: name[QUILocale.getCurrent()]
                    })
                );

                self.$ActiveSwitch.enable();

                if (Location.active) {
                    self.$ActiveSwitch.setAttributes({
                        text  : QUILocale.get(lg, 'controls.locations.location.active.on'),
                        status: 1
                    });

                    self.$ActiveSwitch.setSilentOn();
                } else {
                    self.$ActiveSwitch.setAttributes({
                        text  : QUILocale.get(lg, 'controls.locations.location.active.off'),
                        status: 0
                    });

                    self.$ActiveSwitch.setSilentOff();
                }

                self.Loader.hide();
            });
        },

        /**
         * Panel event: onDestroy
         */
        $onDestroy: function () {
            if (this.$isLocked) {
                return;
            }

            var self = this;

            this.Loader.show();

            this.$unlock().then(function () {
                self.destroy();
            }, function () {
                self.destroy();
            });
        },

        /**
         * Show info category with general location information
         */
        loadInfoCategory: function () {
            // determine address data
            var street   = '',
                location = '',
                country  = '';

            if (this.$Location.attributes.street ||
                this.$Location.attributes.houseNumber) {
                street = this.$Location.attributes.street +
                    ' ' + this.$Location.attributes.houseNumber;
            }

            if (this.$Location.attributes.zipCode ||
                this.$Location.attributes.city) {
                location = this.$Location.attributes.zipCode +
                    ' ' + this.$Location.attributes.city;
            }

            if (this.$Location.attributes.country) {
                country = QUILocale.get(
                    'quiqqer/countries',
                    'country.' + this.$Location.attributes.country
                );
            }

            var Name = JSON.decode(this.$Location.attributes.name);

            this.getBody().set(
                'html',
                Mustache.render(infoTemplate, {
                    labelId     : "ID",
                    id          : this.$Location.id,
                    infoHeader  : QUILocale.get(lg, 'location') + ' "' + Name[QUILocale.getCurrent()] + '"',
                    labelEdit   : QUILocale.get(lg, 'controls.locations.location.info.label.edit'),
                    labelCreate : QUILocale.get(lg, 'controls.locations.location.info.label.create'),
                    labelAddress: QUILocale.get(lg, 'location'),
                    edit        : this.$Location.e_date + ' - ' + this.$Location.e_user,
                    create      : this.$Location.c_date + ' - ' + this.$Location.c_user,
                    street      : street,
                    location    : location,
                    country     : country
                })
            );

            if (this.$Location.attributes.mainImage) {
                new Element('img', {
                    src: window.location.origin + '/' + this.$Location.attributes.mainImage
                }).inject(
                    this.getBody().getElement(
                        '.quiqqer-locations-info-image'
                    )
                );
            }

            this.Loader.hide();

            this.$CurrentCategory = null;
        },

        /**
         * Request the category
         *
         * @param {Object} Category - qui/controls/buttons/Button
         */
        loadCategory: function (Category) {
            var self = this;

            this.Loader.show();

            QUIAjax.get('package_quiqqer_locations_ajax_attributes_getCategoryTemplate', function (result) {
                self.$CurrentCategory = Category;

                var Body = self.getBody();

                if (!result) {
                    result = '';
                }

                Body.set(
                    'html',
                    '<form class="qui-xml-panel">' + result + '</form>'
                );

                // set the form
                var i, len, Elm, value;

                var Form     = Body.getElement('form'),
                    elements = Form.elements;

                Form.setStyle('display', 'none');

                for (i = 0, len = elements.length; i < len; i++) {
                    Elm = elements[i];
                    Elm.addClass('quiqqer-locations-attribute');

                    value = QUIObjectUtils.getValue(Elm.name, self.$Location.attributes);

                    if (!value) {
                        continue;
                    }

                    if (Elm.type == 'checkbox' || Elm.type == 'radio') {
                        Elm.checked = (value).toInt();
                        continue;
                    }

                    Elm.value = value;
                }

                if (self.$locationRefreshMode) {
                    self.$locationRefreshMode = false;
                }

                // parse controls
                Promise.all([

                    QUI.parse(Body),
                    ControlUtils.parse(Body)

                ]).then(function () {

                    Form.setStyle('display', null);

                    var i, len, Node, Control, nodeName;
                    var quiElements = Body.getElements('[data-quiid]');

                    for (i = 0, len = quiElements.length; i < len; i++) {
                        Node     = quiElements[i];
                        nodeName = Node.nodeName;

                        if (nodeName != 'INPUT' &&
                            nodeName != 'TEXTAREA' &&
                            nodeName != 'SELECT') {
                            continue;
                        }

                        Control = QUI.Controls.getById(Node.get('data-quiid'));

                        if (!Control) {
                            continue;
                        }

                        self.$Control = Control;

                        Control.setAttribute('Location', self.$Location);
                        Control.setAttribute('Panel', self);

                        if ("setValue" in Control) {
                            Control.setValue(self.$Location.attributes[Node.get('name')]);
                        }

                        //if (!(Node.get('name') in self.$Location.attributes)) {
                        //    continue;
                        //}
                    }


                    // @todo next part still necessary?

                    // require?
                    if (!Category.getAttribute('require')) {
                        self.Loader.hide();
                        return;
                    }

                    require([Category.getAttribute('require')], function (R) {
                        var type = typeOf(R);

                        if (type === 'function') {
                            R(self);

                        } else if (type === 'class') {
                            self.$Control = new R(self);

                            if (self.getContent().get('html') === '') {
                                self.$Control.inject(Form);

                            } else {
                                self.$Control.imports(Form);
                            }
                        }

                        self.Loader.hide();
                    }, function (err) {
                        QUI.getMessageHandler(function (MH) {
                            MH.addAttention(
                                'Some error occured. Control could not be loaded: ' +
                                Category.getAttribute('require')
                            );

                            console.error(err);
                        });

                        self.Loader.hide();
                    });
                });

            }, {
                'package': 'quiqqer/locations',
                category : Category.getAttribute('name')
            });
        },

        /**
         * Unload the Category and set all settings
         *
         * @param {Boolean} [clear] - Clear the html, default = true
         */
        unloadCategory: function (clear) {
            var i, len, Elm, name, tok, namespace;

            if (typeof clear === 'undefined') {
                clear = true;
            }

            var Body   = this.getBody(),
                Form   = Body.getElement('form'),
                values = {};

            if (Form) {
                var attributeInputs = Form.getElements('.quiqqer-locations-attribute');

                for (i = 0, len = attributeInputs.length; i < len; i++) {
                    Elm  = attributeInputs[i];
                    name = Elm.name;

                    if (name === '') {
                        continue;
                    }


                    if (Elm.type == 'radio' || Elm.type == 'checkbox') {
                        if (Elm.checked) {
                            values[name] = 1;
                        } else {
                            values[name] = 0;
                        }

                        continue;
                    }

                    values[name] = Elm.value;
                }
            }

            // set the values to the $Location object
            for (namespace in values) {
                if (!values.hasOwnProperty(namespace)) {
                    continue;
                }

                this.$Location.attributes[namespace] = values[namespace];
            }

            if (this.$Control && clear) {
                this.$Control.destroy();
                this.$Control = null;
            }
        },

        /**
         * event : click on category button
         *
         * @param {Object} Category - qui/controls/buttons/Button
         */
        $onCategoryActive: function (Category) {
            if (!this.$locationRefreshMode) {
                this.unloadCategory();
            }

            this.loadCategory(Category);
        },

        /**
         * Send the configuration to the server
         */
        save: function () {
            this.fireEvent('save', [this]);

            this.unloadCategory(false);

            var self   = this;
            var inList = {};

            // filter controls with save method
            var saveable = QUI.Controls.getControlsInElement(this.getBody())
                .filter(function (Control) {
                    if (Control.getId() in inList) {
                        return false;
                    }

                    if (typeof Control.save === 'undefined') {
                        return false;
                    }

                    inList[Control.getId()] = true;
                    return true;
                });

            var promises = saveable.map(function (Control) {
                return Control.save();
            }).filter(function (Promise) {
                return typeof Promise.then == 'function';
            });

            if (!promises.length) {
                promises = [Promise.resolve()];
            }

            this.Loader.show();

            Promise.all(promises).then(function () {
                var Save = this.getButtonBar().getElement('save');

                Save.setAttribute('textimage', 'fa fa-refresh fa-spin');

                QUIAjax.post('package_quiqqer_locations_ajax_locations_update', function (result) {
                    Save.setAttribute('textimage', 'fa fa-save');

                    var name = JSON.decode(self.$Location.attributes.name);

                    self.setAttribute(
                        'title',
                        QUILocale.get(lg, 'controls.locations.location.title', {
                            locationId  : self.$Location.id,
                            locationName: name[QUILocale.getCurrent()]
                        })
                    );

                    self.$locationRefresh().then(function () {
                        if (!self.$CurrentCategory) {
                            self.loadInfoCategory();
                        } else {
                            self.loadCategory(self.$CurrentCategory);
                        }
                    });

                    self.Loader.hide();
                }, {
                    'package' : 'quiqqer/locations',
                    locationId: this.$Location.id,
                    attributes: JSON.encode(this.$Location.attributes)
                });

            }.bind(this));
        },

        /**
         * onEvent: user clicks "(de)activate" switch
         */
        $onActivationStatusChange: function () {
            var self   = this;
            var Button = this.getButtons('status');

            Button.disable();
            this.Loader.show();

            if (Button.getStatus()) {
                Locations.activateLocation(this.$Location.id).then(function (success) {
                    self.Loader.hide();
                    Button.enable();

                    if (!success) {
                        Button.setSilentOff();
                        return;
                    }

                    Button.setAttribute(
                        'text',
                        QUILocale.get(lg, 'controls.locations.location.active.on')
                    );
                });

                return;
            }

            Locations.deactivateLocation(this.$Location.id).then(function (success) {
                self.Loader.hide();
                Button.enable();

                if (!success) {
                    Button.setSilentOn();
                }

                Button.setAttribute(
                    'text',
                    QUILocale.get(lg, 'controls.locations.location.active.off')
                );
            });
        },

        /**
         * Check if this location is currently edited by another user
         */
        $checkLock: function () {
            if (!this.$lockKey || !this.$initialLoad) {
                return;
            }

            var self = this;

            this.Loader.show();

            QUILocker.isLocked(this.$lockKey, 'quiqqer/locations').then(function (locked) {
                self.Loader.hide();

                if (locked !== false) {
                    self.$isLocked = true;
                    self.$showLockInfo();
                    return;
                }

                self.$isLocked = false;
                self.$lock();
            });
        },

        /**
         * Lock panel for current session user
         *
         * @returns {Promise}
         */
        $lock: function () {
            var self = this;

            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_locations_lock', resolve, {
                    'package' : 'quiqqer/locations',
                    locationId: self.getAttribute('locationId'),
                    lockKey   : self.$lockKey,
                    onError   : reject
                });
            });
        },

        /**
         * Unlock panel
         *
         * @returns {Promise}
         */
        $unlock: function () {
            var self = this;

            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_locations_ajax_locations_unlock', resolve, {
                    'package' : 'quiqqer/locations',
                    locationId: self.getAttribute('locationId'),
                    lockKey   : self.$lockKey,
                    onError   : reject
                });
            });
        },

        /**
         * Shows lock info
         */
        $showLockInfo: function () {
            var self = this;

            QUIAjax.get('package_quiqqer_locations_ajax_locations_canUnlock', function (canUnlock) {
                var lockInfo = '';

                if (canUnlock) {
                    lockInfo = QUILocale.get(lg, 'controls.locations.location.lock.unlock.info');
                } else {
                    lockInfo = QUILocale.get(lg, 'controls.locations.location.lock.info');
                }

                var LockInfoElm = new Element('div', {
                    'class': 'quiqqer-locations-location-lock-info',
                    html   : '<span class="fa fa-lock quiqqer-locations-location-lock-info-icon"></span>' +
                        '<h1>' + QUILocale.get(lg, 'controls.locations.location.lock.title') + '</h1>' +
                        '<p>' + lockInfo + '</p>' +
                        '<div class="quiqqer-locations-location-lock-btn"></div>'
                }).inject(self.$Elm);

                if (canUnlock) {
                    new QUIButton({
                        text     : QUILocale.get(lg, 'controls.locations.location.lock.btn.ignore.text'),
                        textimage: 'fa fa-unlock',
                        events   : {
                            onClick: function () {
                                self.Loader.show();

                                self.$lock().then(function (success) {
                                    self.Loader.hide();

                                    if (success) {
                                        self.$isLocked = false;
                                        LockInfoElm.destroy();
                                    }
                                }, function () {
                                    self.destroy();
                                });
                            }
                        }
                    }).inject(
                        LockInfoElm.getElement('.quiqqer-locations-location-lock-btn')
                    );
                }

                new QUIButton({
                    text     : QUILocale.get(lg, 'controls.locations.location.lock.btn.close.text'),
                    textimage: 'fa fa-close',
                    events   : {
                        onClick: function () {
                            self.destroy();
                        }
                    }
                }).inject(
                    LockInfoElm.getElement('.quiqqer-locations-location-lock-btn')
                );
            }, {
                'package': 'quiqqer/locations'
            });
        }
    });
});
