/**
 * LocatorFilters main class
 *
 * Collects data from Location filter controls
 * Open and close fil
 *
 * @module package/quiqqer/locations/bin/frontend/controls/Locator/LocatorFilters
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onChange [FilterData, this] - Fires if any Filter control changes its values
 */
define('package/quiqqer/locations/bin/frontend/controls/Locator/LocatorFilters', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/loader/Loader',

    'utils/Controls',
    'URI',

    'package/quiqqer/locations/bin/frontend/Locator'

], function (QUI, QUIControl, QUILoader, QUIControlUtils, URI, Locator) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/frontend/controls/Locator/LocatorFilters',

        Binds: [
            '$onImport',
            'getFilterData',
            '$onFilterChange',
            'openFilters',
            'closeFilters',
            'toggleFilters',
            'resetFilters'
        ],

        initialize: function (options) {
            this.parent(options);

            this.Loader          = new QUILoader();
            this.$Content        = null;
            this.$FilterControls = {};
            this.filterOpen      = false;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var Elm  = this.getElm(),
                self = this;

            this.Loader.inject(Elm);

            // Parse filter controls
            this.Loader.show();

            var filterElements = Elm.getElements('.quiqqer-locations-locator-filter'),
                filterPromises = [];

            for (var i = 0, len = filterElements.length; i < len; i++) {
                var FilterElm = filterElements[i];

                if (FilterElm.get('data-qui')) {
                    filterPromises.push(QUIControlUtils.getControlByElement(FilterElm));
                }
            }

            if (Elm.get('data-qui-slideoutfilter')) {
                this.initSlideOutFilter();
            }

            Promise.all(filterPromises).then(function (result) {
                for (var i = 0, len = result.length; i < len; i++) {
                    var FilterControl = result[i];
                    var idHash        = FilterControl.getElm().getParent().get('data-hash');

                    self.$FilterControls[idHash] = FilterControl;

                    FilterControl.addEvent('onChange', self.$onFilterChange);
                }

                self.Loader.hide();
            });
        },

        initSlideOutFilter: function () {
            // toggle filters
            var ToggleFilters = document.getElement('.quiqqer-locations-locator-toggleFilter');

            if (ToggleFilters) {
                ToggleFilters.addEvent('click', this.toggleFilters)
            }

            // close filters
            new Element('span', {
                'class': 'fa fa-close quiqqer-locations-locator-filter-close',
                events : {
                    click: this.closeFilters
                }
            }).inject(this.getElm());
        },

        /**
         * Triggers if any Filter control changes
         */
        $onFilterChange: function () {
            this.fireEvent('change', [this.getFilterData(), this]);
        },

        /**
         * Get filter data of all Filter controls
         *
         * @return {Object}
         */
        getFilterData: function () {
            var FilterData = {};

            for (var filterHash in this.$FilterControls) {
                if (!this.$FilterControls.hasOwnProperty(filterHash)) {
                    continue;
                }

                if ("getFilterData" in this.$FilterControls[filterHash]) {
                    FilterData[filterHash] = this.$FilterControls[filterHash].getFilterData();
                }
            }

            return FilterData;
        },

        /**
         * Toggle filter menu
         */
        toggleFilters: function () {
            if (this.filterOpen) {
                this.closeFilters();
                return;
            }

            this.openFilters();
        },

        /**
         * Open filter
         */
        openFilters: function () {
            if (this.filterOpen) {
                return;
            }

            this.filterOpen = true;

            var Elm = this.getElm();

            Elm.setStyle('display', '');

            moofx(this.getElm()).animate({
                left: 0
            }, {
                duration: 300,
            });
        },

        /**
         * Close filters
         */
        closeFilters: function () {
            if (!this.filterOpen) {
                return;
            }

            this.filterOpen = false;

            var Elm   = this.getElm(),
                width = Elm.getSize().x;

            moofx(Elm).animate({
                left: -width - 10 // 10 because of Elm shadow
            }, {
                duration: 300,
                callback: function () {
                    Elm.setStyle('display', 'none');
                }
            });
        },

        /**
         * Reset all filters
         *
         * @return {Promise}
         */
        resetFilters: function () {
            var resetPromises = [];

            for (var filterHash in this.$FilterControls) {
                if (!this.$FilterControls.hasOwnProperty(filterHash)) {
                    continue;
                }

                if ("reset" in this.$FilterControls[filterHash]) {
                    resetPromises.push(this.$FilterControls[filterHash].reset());
                }
            }

            return Promise.all(resetPromises);
        }
    });
});
