/**
 * LocatorMap main class
 *
 * Shows location results on map and displays info layers with location details
 *
 * @module package/quiqqer/locations/bin/frontend/controls/Locator/LocatorMap
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event mapLoaded [this] - Fires if the map finished loading
 */
define('package/quiqqer/locations/bin/frontend/controls/Locator/LocatorMap', [

    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'Locale',

    'package/quiqqer/geoutils/bin/Geo',
    'package/quiqqer/geoutils/bin/classes/GeoMapMarker',
    'package/quiqqer/geoutils/bin/classes/GeoPoint',

    'package/quiqqer/locations/bin/frontend/Locator'

], function (QUIControl, QUILoader, QUILocale, GeoApi, GeoMapMarker, GeoPoint, Locator) {
    "use strict";

    var lg = 'quiqqer/locations';

    return new Class({
        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/frontend/controls/Locator/LocatorMap',

        Binds: [
            '$onImport',
            '$showViewBox',
            '$populateMap',
            '$onMapMarkerClick',
            'openLocationInfoWindow',
            '$showAllLocations',
            '$showViewCenter'
        ],

        options: {
            layershowimages   : true,
            infowindowoffsety : 0,
            showonemptyresults: 'all',          // determines what is displayed if the search results are empty
            maptype           : 'roadmap',      // default map type
            locations         : false,          // special server-side attribute; show these location initially (base64 encoded JSON only)
            viewbox           : false,          // special server-side attribute; show this viewbox initially (base64 encoded JSON only)
            center            : false,          // special server-side attribute; show this center initially (base64 encoded JSON only)
            markericon        : false,          // false = use map provider default map marker icon
            usericon          : false           // false = use map provider default map marker icon
        },

        initialize: function (options) {
            this.parent(options);

            this.Loader           = new QUILoader();
            this.$Map             = null;
            this.$locations       = [];
            this.$ViewBox         = false;
            this.$ViewCenter      = false;
            this.$LocationMarkers = {};

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var self = this;
            var Elm  = this.getElm();

            this.Loader.inject(Elm);
            this.Loader.show();

            Elm.set('html', '');

            GeoApi.getMap({
                maptype: self.getAttribute('maptype')
            }).then(function (Map) {
                self.$Map = Map;
                self.$Map.inject(Elm);

                self.$Map.addEvents({
                    onLoaded      : function () {
                        // Check for pre-defined location list
                        var locations = self.getAttribute('locations'),
                            viewbox   = self.getAttribute('viewbox'),
                            center    = self.getAttribute('center');

                        if (!locations) {
                            self.$showAllLocations().then(function () {
                                self.fireEvent('mapLoaded', [self]);
                            });

                            return;
                        }

                        self.$locations = JSON.decode(atob(locations));
                        self.$populateMap();

                        if (viewbox) {
                            self.$ViewBox = JSON.decode(atob(viewbox));
                            self.$showViewBox();
                        } else if (center) {
                            self.$ViewCenter = JSON.decode(atob(center));
                            self.$showViewCenter();
                        }

                        self.fireEvent('mapLoaded', [self]);
                    },
                    onMapLoadError: self.$onMapLoadError,
                    onMarkerClick : self.$onMapMarkerClick
                });
            }, this.$onMapLoadError);
        },

        /**
         * Show info message if map could not be loaded!
         */
        $onMapLoadError: function () {
            var Elm = this.getElm();

            Elm.set('html', '');

            new Element('div', {
                'class': 'quiqqer-locations-locator-map-error',
                html   : QUILocale.get(lg, 'controls.Locator.LocatorMap.load_error')
            }).inject(Elm);

            this.Loader.hide();
        },

        /**
         * Show all available locations on the map
         *
         * @param {Object} [FilterData]
         * @param {Boolean} [updateViewBox] - [default: true]
         * @return {Promise}
         */
        $showAllLocations: function (FilterData, updateViewBox) {
            var self = this;

            if (typeof updateViewBox === 'undefined') {
                updateViewBox = true;
            }

            this.Loader.show();

            return Locator.getAllLocationPoints(FilterData).then(function (Result) {
                self.$locations  = Result.locations;
                self.$ViewCenter = Result.viewCenter;

                if (self.$locations.length) {
                    self.$populateMap();
                } else {
                    if (self.getAttribute('showonemptyresults') === 'all') {
                        self.$showAllLocations();
                    }

                    self.$Map.removeMarkers();
                }

                if (updateViewBox) {
                    self.$showViewCenter();
                }

                self.Loader.hide();
            });
        },

        /**
         * Perform a location search based on a center geo location
         *
         * @param {package/quiqqer/geoutils/bin/classes/GeoPoint} GeoCenter
         * @param {Number} radius
         * @param {Object} [FilterData]
         * @param {Boolean} [updateViewBox] - Default: true
         */
        search: function (GeoCenter, radius, FilterData, updateViewBox) {
            var self = this;

            if (typeof updateViewBox === 'undefined') {
                updateViewBox = true;
            }

            if (!GeoCenter) {
                this.$showAllLocations(FilterData, updateViewBox);
                return;
            }

            this.Loader.show();

            Locator.getMapLocationList(GeoCenter.lat(), GeoCenter.lng(), radius, FilterData).then(function (Result) {
                self.$locations = Result.locations;
                self.$ViewBox   = Result.viewBox;

                self.$Map.removeMarkers();

                if (self.$locations.length) {
                    if (self.$locations.length > 1 && updateViewBox) {
                        self.$showViewBox();
                    }

                    self.$populateMap();
                } else {
                    self.$Map.showCoordinates(GeoCenter);
                }

                // Set marker at user position
                //self.$Map.addMarker(new GeoMapMarker({
                //    position      : GeoCenter,
                //    isUserLocation: true
                //}));

                self.Loader.hide();
            });
        },

        /**
         * Open the info windows for a specific location
         *
         * @param locationId
         */
        openLocationInfoWindow: function (locationId) {
            if (typeof this.$LocationMarkers[locationId] === 'undefined') {
                return;
            }

            this.$onMapMarkerClick(this.$LocationMarkers[locationId]);
        },

        /**
         * Event: onMarkerClick
         *
         * Handle clicks on location markers on the map
         *
         * @param {package/quiqqer/geoutils/bin/classes/GeoMapMarker} GeoMapMarker
         */
        $onMapMarkerClick: function (GeoMapMarker) {
            var self = this;

            this.$Map.hideInfoWindow();

            if (GeoMapMarker.getAttribute('isUserLocation')) {
                this.$Map.showInfoWindow(
                    GeoMapMarker.getPosition(),
                    QUILocale.get(lg, 'controls.Locator.LocatorMap.user_marker_info'),
                    this.getAttribute('infowindowoffsety')
                );

                return;
            }

            // Show Loader in info window
            this.$Map.showInfoWindow(
                GeoMapMarker.getPosition(),
                '<span class="fa fa-spinner fa-spin"></span>',
                this.getAttribute('infowindowoffsety')
            );

            // Load location details
            Locator.getMapLocationDetails(GeoMapMarker.getAttribute('id')).then(function (mapLocationDetails) {
                self.$Map.hideInfoWindow();

                // Show location details in info window
                self.$Map.showInfoWindow(
                    GeoMapMarker.getPosition(),
                    mapLocationDetails,
                    self.getAttribute('infowindowoffsety')
                );
            });
        },

        /**
         * Show bounding box based on all currently displayed locations
         */
        $showViewBox: function () {
            this.$Map.showBox(
                this.$ViewBox.northLat,
                this.$ViewBox.southLat,
                this.$ViewBox.eastLng,
                this.$ViewBox.westLng
            );
        },

        /**
         * Show view center
         */
        $showViewCenter: function () {
            if (this.$ViewCenter) {
                this.$Map.showCoordinates(new GeoPoint({
                    lat: this.$ViewCenter.lat,
                    lng: this.$ViewCenter.lng
                }));

                this.$Map.setZoom(this.$ViewCenter.zoom);
            }
        },

        /**
         * Populate map with location markers
         */
        $populateMap: function () {
            this.$LocationMarkers = {};

            this.$Map.removeMarkers();

            var singleLocation = this.$locations.length === 1;

            for (var i = 0, len = this.$locations.length; i < len; i++) {
                var Location = this.$locations[i];

                var Marker = new GeoMapMarker(Location);
                Marker.setAttribute('icon', Location.map_icon);

                this.$LocationMarkers[Location.id] = Marker;
                this.$Map.addMarker(Marker);

                if (singleLocation) {
                    this.$Map.showCoordinates(new GeoPoint(Location));
                    this.$Map.setZoom(15);
                }
            }
        }
    });
});
