/**
 * Search locations based on zipcode and category
 *
 * @authro www.pcsg.de (Patrick Müller)
 * @module package/quiqqer/locations/bin/frontend/controls/Search/LocationSearch
 *
 * @event onLoaded [this] - Fires when category data has been loaded
 */
define('package/quiqqer/locations/bin/frontend/controls/Search/LocationSearch', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'utils/Controls',

    'qui/utils/Functions',
    'qui/controls/input/Suggest',
    'package/quiqqer/geoutils/bin/Geo',

    'Ajax',
    'Locale',

    'css!package/quiqqer/locations/bin/frontend/controls/Search/LocationSearch.css'

], function (QUI, QUIControl, QUILoader, QUIControlUtils, QUIFunctionUtils, QUISuggest, GeoApi, QUIAjax, QUILocale) {
    "use strict";

    var lg = 'quiqqer/locations';

    /**
     * @class package/quiqqer/locations/bin/frontend/controls/Search/LocationSearch
     *
     * @param {number} locationId - ID of LocationSearch
     * @param {string} [icon] - Panel icon
     */
    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/locations/bin/frontend/controls/Search/LocationSearch',

        Binds: [
            '$onImport',
            '$suggestSearch'
        ],

        options: {
            categoryid: false,
            country   : false   // search country
        },

        initialize: function (options) {
            this.parent(options);

            this.$Pagination        = null;
            this.Loader             = new QUILoader();
            this.$SearchStringInput = null;
            this.$NoResults         = null;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var self = this,
                Elm  = this.getElm();

            this.Loader.inject(Elm);
            //this.Loader.show();

            //// Init pagination
            //QUIControlUtils.getControlByElement(
            //    Elm.getElement('[data-qui="package/quiqqer/controls/bin/navigating/Pagination"]')
            //).then(function (PaginationControl) {
            //    self.$Pagination = PaginationControl;
            //
            //    self.$Pagination.addEvents({
            //        onChange: self.$onPaginationChange
            //    });
            //
            //    self.Loader.hide();
            //}, function () {
            //    // no pagination
            //    self.Loader.hide();
            //});

            // Init suggest search
            var Form               = Elm.getElement('form');
            var SuggestInput       = Elm.getElement('.quiqqer-locations-search-locations-search > input');
            var GeoIdentifierInput = Elm.getElement('input[name="geo_identifier"]');
            var SubmitBtn          = Elm.getElement('button.quiqqer-locations-search-locations-submit');
            var onSuggestChange    = QUIFunctionUtils.debounce(this.$suggestSearch, 500);

            this.$SearchStringInput = Elm.getElement('input[name="search_string"]');

            SubmitBtn.addEvent('click', function () {
                var SuggestInput = self.$Suggest.getInput();
                var suggestValue = SuggestInput.value.trim();

                if (suggestValue === '') {
                    SuggestInput.focus();
                    return;
                }

                self.$suggestSearch().then(function () {
                    self.$Suggest.select();
                });
            });

            Form.addEvent('submit', function (event) {
                event.stop();
            });

            this.$Suggest = new QUISuggest({
                autosearch: false,
                events    : {
                    onChange: onSuggestChange,
                    onSelect:
                        function (data) {
                            var SuggestInput = self.$Suggest.getInput();

                            if (SuggestInput.value.trim() === '') {
                                GeoIdentifierInput.value = '';
                                self.$Suggest.clearOptions();
                            } else {
                                GeoIdentifierInput.value = data.value;
                            }

                            self.$SearchStringInput.value = data.text;

                            Form.submit();
                        }
                }
            }).imports(SuggestInput);

            SuggestInput.disabled = false;
        },

        /**
         * On suggest search
         *
         * @return {Promise}
         */
        $suggestSearch: function () {
            var self        = this,
                searchValue = this.$Suggest.getInput().value.trim(),
                country     = this.getAttribute('country');

            this.$geoIdentifier = null;

            if (searchValue.length < 3) {
                return;
            }

            this.$Suggest.getInput().value = searchValue;
            this.$Suggest.showLoader();

            return GeoApi.getSuggestionListBySearchString(searchValue, country).then(function (results) {
                self.$Suggest.clearOptions();

                if (!results.length) {
                    var SuggestInput = self.$Suggest.getInput();

                    if (self.$NoResults) {
                        self.$NoResults.destroy();
                    }

                    var NoResults = new Element('div', {
                        'class': 'quiqqer-locations-locationsearch-no-results',
                        html   : QUILocale.get(lg, 'controls.Locator.LocatorSearch.no_results'),
                        styles : {
                            position: 'fixed',
                            top     : SuggestInput.getPosition().y + SuggestInput.getSize().y
                        }
                    }).inject(self.$Suggest.getInput(), 'after');

                    self.$NoResults = NoResults;

                    self.$Suggest.hideLoader();

                    (function () {
                        if (NoResults) {
                            NoResults.destroy();
                        }
                    }).delay(2000);
                    return;
                }

                for (var i = 0, len = results.length; i < len; i++) {
                    var suggestion = results[i];

                    self.$Suggest.addOption(
                        suggestion.description,
                        suggestion.identifier
                    );
                }

                self.$Suggest.refreshSuggest();
                self.$Suggest.hideLoader();
                self.$Suggest.showSuggest();
            });
        }
    });
});
