<?php

namespace QUI\Locations\Attributes;

use QUI;
use QUI\Locations\Handler\Attributes as AttributesHandler;
use QUI\Locations\Utils as LocationsUtils;

/**
 * Class Attribute
 *
 * Represents a single location Attribute
 *
 * @package QUI\Locations
 */
abstract class AbstractAttribute
{
    /**
     * Attribute name
     *
     * @var string
     */
    protected $name = null;

    /**
     * Title translation var data
     *
     * @var array
     */
    protected $title = [];

    /**
     * Attribute priority
     *
     * @var int
     */
    protected $priority = null;

    /**
     * Attribute class
     *
     * @var null|string
     */
    protected $attributeClass = null;

    /**
     * Attribute source package
     *
     * @var string
     */
    protected $sourcePackage = null;

    /**
     * Attribute value
     *
     * @var mixed
     */
    protected $value = null;

    /**
     * Attribute constructor.
     *
     * @param string $name - Attribute name
     * @throws QUI\Locations\Exception
     */
    public function __construct($name)
    {
        $result = QUI::getDataBase()->fetch([
            'from'  => AttributesHandler::getLocationAttributesTable(),
            'where' => [
                'name' => $name
            ]
        ]);

        if (empty($result)) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.attribute.not.found',
                [
                    'name' => $name
                ]
            ], 404);
        }

        $attributeData = $result[0];

        $this->name           = $attributeData['name'];
        $this->priority       = $attributeData['priority'];
        $this->attributeClass = $attributeData['attribute_class'];
        $this->sourcePackage  = $attributeData['source_package'];
        $this->title          = json_decode($attributeData['title'], true);
    }

    /**
     * Set a value to the attribute
     *
     * @param mixed $value
     * @throws QUI\Locations\Exception
     */
    public function setValue($value)
    {
        $value = LocationsUtils::clearJSONString($value);
        $this->validate($value);
        $this->value = $this->cleanUpValue($value);
    }

    /**
     * Get raw value of the attribute
     *
     * @throws QUI\Locations\Exception
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * Get unique attribute name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Get (translated) title of attribute
     *
     * @param QUI\Locale $Locale (optional) - if omitted, use QUI::getLocale()
     * @return string
     */
    public function getTitle($Locale = null)
    {
        if (is_null($Locale)) {
            $Locale = QUI::getLocale();
        }

        return $Locale->get(
            $this->title['group'],
            $this->title['var']
        );
    }

    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    abstract public function validate($value);

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    abstract public function getSearchCacheValue($Locale = null);

    /**
     * Get the value that is used in frontend area
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    abstract public function getViewValue($Locale = null);

    /**
     * Cleanup value
     *
     * @param mixed $value
     * @return mixed
     */
    public function cleanUpValue($value)
    {
        return $value;
    }
}
