<?php

namespace QUI\Locations\Attributes\Types;

use QUI;
use QUI\Locations\Attributes\AbstractAttribute;
use QUI\Locations\Handler\Attributes as AttributesHandler;
use QUI\Locations\Utils as LocationsUtils;

/**
 * Class OpeningHours
 *
 * Attribute that contains regular opening hours of a location
 *
 * @package QUI\Locations
 */
class SpecialOpeningHours extends AbstractAttribute
{
    /**
     * Validate the value of an attribute
     *
     * @param string|array $value - The input value (from the frontend)
     * @return bool - true, if value is valid; otherwise an exception is thrown
     *
     * @throws QUI\Locations\Exception
     */
    public function validate($value)
    {
        if (empty($value)) {
            return true;
        }

        if (is_string($value)) {
            $value = json_decode($value, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.json.error'
                ]);
            }
        }

        if (!is_array($value)) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.types.datatype.array.expected'
            ]);
        }

        /** @var OpeningHours $OpeningHours */
        $OpeningHours = AttributesHandler::getLocationAttribute('openingHours');

        foreach ($value as $data) {
            if (!isset($data['time'])
                || !is_array($data['time'])
                || !isset($data['recurrence'])
            ) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.wrong'
                ]);
            }

            // check time entries
            $OpeningHours->validate($data['time']);

            // check recurrence
            $rc = $data['recurrence'];

            if (!isset($rc['type'])
                || !is_string($rc['type'])
                || !isset($rc['options'])
                || !is_array($rc['options'])
            ) {
                throw new QUI\Locations\Exception([
                    'quiqqer/locations',
                    'exception.attributes.types.datatype.wrong'
                ]);
            }

            $options = $rc['options'];

            switch ($rc['type']) {
                case 'daterange':
                    if (!isset($options['from'])
                        || !isset($options['to'])
                    ) {
                        throw new QUI\Locations\Exception([
                            'quiqqer/locations',
                            'exception.attributes.types.datatype.wrong'
                        ]);
                    }

                    $fromParts = explode("-", $options['from']);
                    $toParts   = explode("-", $options['to']);

                    if (count($fromParts) < 3
                        || count($toParts) < 3
                    ) {
                        throw new QUI\Locations\Exception([
                            'quiqqer/locations',
                            'exception.attributes.types.datatype.wrong'
                        ]);
                    }

                    if (!checkdate($fromParts[1], $fromParts[2], $fromParts[0])
                        || !checkdate($toParts[1], $toParts[2], $toParts[0])
                    ) {
                        throw new QUI\Locations\Exception([
                            'quiqqer/locations',
                            'exception.attributes.types.specialopeninghours.wrong.date'
                        ]);
                    }

                    break;
                case 'yearly':
                    if (!isset($options['from'])
                        || !isset($options['to'])
                    ) {
                        throw new QUI\Locations\Exception([
                            'quiqqer/locations',
                            'exception.attributes.types.datatype.wrong'
                        ]);
                    }

                    $fromDay   = mb_substr($options['from'], 3, 2);
                    $fromMonth = mb_substr($options['from'], 0, 2);
                    $toDay     = mb_substr($options['to'], 3, 2);
                    $toMonth   = mb_substr($options['to'], 0, 2);

                    if (!checkdate($fromMonth, $fromDay, 2016)
                        || !checkdate($toMonth, $toDay, 2016)
                    ) {
                        throw new QUI\Locations\Exception([
                            'quiqqer/locations',
                            'exception.attributes.types.specialopeninghours.wrong.date'
                        ]);
                    }

                    break;
                case 'specialday':
                    if (!isset($options['daynum'])
                        || !isset($options['month'])
                    ) {
                        throw new QUI\Locations\Exception([
                            'quiqqer/locations',
                            'exception.attributes.types.datatype.wrong'
                        ]);
                    }

                    switch ($options['daynum']) {
                        case 'every':
                        case 1:
                        case 2:
                        case 3:
                        case 4:
                        case 5:
                        case 'last':
                            break;

                        default:
                            throw new QUI\Locations\Exception([
                                'quiqqer/locations',
                                'exception.attributes.types.datatype.wrong'
                            ]);
                    }

                    switch ($options['month']) {
                        case 'month':
                        case 1:
                        case 2:
                        case 3:
                        case 4:
                        case 5:
                        case 6:
                        case 7:
                        case 8:
                        case 9:
                        case 10:
                        case 11:
                        case 12:
                            break;

                        default:
                            throw new QUI\Locations\Exception([
                                'quiqqer/locations',
                                'exception.attributes.types.datatype.wrong'
                            ]);
                    }

                    break;

                default:
                    throw new QUI\Locations\Exception([
                        'quiqqer/locations',
                        'exception.attributes.types.datatype.wrong'
                    ]);
            }
        }

        return true;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getSearchCacheValue($Locale = null)
    {
        return $this->value;
    }

    /**
     * Get the value that is used in the search cache table for this attribute
     *
     * @param QUI\Locale $Locale (optional) - Value depending on specific Locale (language);
     * if omitted QUI\Locations\Handler\Locations::getLocale is used
     * @return string - Database value
     */
    public function getViewValue($Locale = null)
    {
        if (empty($this->value)) {
            return $this->value;
        }

        // determine if location is currently open
        $value = json_decode($this->value, true);

        if (empty($value)) {
            return $value;
        }

        $open         = false;
        $openingLines = [];

        foreach ($value as $k => $entry) {
            $openTime = false;

            foreach ($entry['time'] as $timeEntry) {
                if (LocationsUtils::isOpen($timeEntry)) {
                    $openTime = true;
                    break;
                }
            }

            $openRecurrence   = $this->isRecurrenceNow($entry['recurrence']);
            $entry['openNow'] = $openTime && $openRecurrence;

            if ($entry['openNow']) {
                $open = true;
            }

            $openingLine = [
                'recurrence' => LocationsUtils::convertSpecialOpeningRecurrence($entry['recurrence']),
                'lines'      => []
            ];

            foreach ($entry['time'] as $time) {
                $openingLine['lines'][] = LocationsUtils::convertOpeningDataToText($time);
            }

            $openingLines[] = $openingLine;

            $value[$k] = $entry;
        }

        $value['openingLines'] = $openingLines;
        $value['openNow']      = $open;

        return $value;
    }

    /**
     * Checks if given recurrence information applies to the current date
     *
     * @param array $recurrenceData
     * @return bool - true if current date is in the boundaries of recurrence date range
     */
    protected function isRecurrenceNow($recurrenceData)
    {
        $time = $recurrenceData['options'];

        switch ($recurrenceData['type']) {
            case 'daterange':
                $dateNow  = time();
                $dateFrom = strtotime($time['from']);
                $dateTo   = strtotime($time['to']) + 85399; // 23:59

                if ($dateFrom <= $dateNow
                    && $dateTo >= $dateNow
                ) {
                    return true;
                }
                break;

            case 'yearly':
                $fromParts = explode('-', $time['from']);
                $monthFrom = (int)$fromParts[0];
                $dayFrom   = (int)$fromParts[1];

                $toParts = explode('-', $time['to']);
                $monthTo = (int)$toParts[0];
                $dayTo   = (int)$toParts[1];

                $monthNow = date('n');
                $dayNow   = date('j');

                if ($monthFrom <= $monthNow
                    && $monthTo >= $monthNow
                ) {
                    if ($monthFrom == $monthTo) {
                        if ($dayFrom <= $dayNow
                            && $dayTo >= $dayNow
                        ) {
                            return true;
                        }
                    } else {
                        if ($monthFrom == $monthNow) {
                            return $dayNow >= $dayFrom;
                        } else {
                            return $dayNow <= $dayTo;
                        }
                    }
                }
                break;

            case 'specialday':
                $dayNum = $time['daynum'];
                $month  = $time['month'];

                if ($dayNum == 'every'
                    && $month == 'month'
                ) {
                    return true;
                }

                $monthNow = date('n');

                if ($monthNow != (int)$month) {
                    return false;
                }

                if ($dayNum == 'every') {
                    return true;
                }

                switch ($dayNum) {
                    case 1:
                        $dayNum = 'first';
                        break;

                    case 2:
                        $dayNum = 'second';
                        break;

                    case 3:
                        $dayNum = 'third';
                        break;

                    case 4:
                        $dayNum = 'fourth';
                        break;
                }

                $dateQueryText = $dayNum." ".date("l")." of ".date("F Y");
                $dateTimeStamp = strtotime($dateQueryText);

                if (time() >= $dateTimeStamp
                    && time() <= ($dateTimeStamp + 85399)
                ) {
                    return true;
                }
                break;

            default:
                return false;
        }

        return false;
    }
}
