<?php

namespace QUI\Locations\Controls;

use QUI;
use QUI\Locations\Locator;
use QUI\Controls\Navigating\Pagination;

/**
 * Class LocatorList
 *
 * Main control for Locator search results as a list
 */
class LocatorListResult extends QUI\Control
{
    /**
     * constructor
     *
     * @param array $attributes
     */
    public function __construct($attributes = [])
    {
        $this->setAttributes([
            'Site'          => false,
            'Center'        => false,
            'radius'        => 25.0,
            'page'          => 1,
            'perPage'       => 10,
            'listOnly'      => false,  // do not include Pagination
            'locationIds'   => false,  // restrict results to these location IDs
            'locationImage' => false // show main image
        ]);

        $this->addCSSClass('quiqqer-locations-locator-list');

        parent::__construct($attributes);
    }

    /**
     * (non-PHPdoc)
     *
     * @throws QUI\Exception
     * @see \QUI\Control::create()
     *
     */
    public function getBody()
    {
        $Engine = QUI::getTemplateManager()->getEngine();

        $results    = $this->getResults();
        $listOnly   = $this->getAttribute('listOnly');
        $Pagination = false;

        if (!$listOnly) {
            $Pagination = new Pagination([
                'limit'   => $results['perPage'],
                'count'   => $results['total'],
                'sheet'   => $results['page'],
                'useAjax' => true,
                'Site'    => $this->getSite()
            ]);
        }

        $Engine->assign([
            'locations'  => $results['list'],
            'listOnly'   => $listOnly,
            'Pagination' => $Pagination,
            'locationImage' => $this->getAttribute('locationImage'),
            'css'        => \file_get_contents(\dirname(__FILE__) . '/LocatorListResult.css')
        ]);

        return $Engine->fetch(\dirname(__FILE__) . '/LocatorListResult.html');
    }

    /**
     * Get search results
     *
     * @return array
     * @throws QUI\Exception
     */
    protected function getResults()
    {
        $Site    = $this->getSite();
        $perPage = $this->getAttribute('perPage');

        if ($Site->getAttribute('quiqqer.locations.locator.list.results_per_page')) {
            $perPage = (int)$Site->getAttribute('quiqqer.locations.locator.list.results_per_page');
        }

        $page = (int)$this->getAttribute('page');

        if ($page < 1) {
            $page = 1;
        }

        $radius = (float)$this->getAttribute('radius');

        if ($radius < 10) {
            $radius = 10.0;
        } elseif ($radius > 200) {
            $radius = 200.0;
        }

        /** @var QUI\Geo\Utils\GeoPoint $Center */
        $Center = $this->getAttribute('Center');

        $searchParams = [
            'perPage' => $perPage,
            'page'    => $page
        ];

        // Filter by location IDs
        $locationIds = $this->getAttribute('locationIds');

        if ($locationIds !== false) {
            $searchParams['allowedIds'] = $locationIds;
        }

        $list = Locator::getLocationsInRadius(
            $Center,
            $searchParams,
            $radius
        );

        $count = Locator::getLocationsInRadius(
            $Center,
            [
                'count' => true
            ],
            $radius
        );

        return [
            'list'    => $list,
            'page'    => $page,
            'perPage' => $perPage,
            'total'   => $count
        ];
    }

    /**
     * Return the current site
     *
     * @return mixed|QUI\Projects\Site
     *
     * @throws QUI\Exception
     */
    protected function getSite()
    {
        if ($this->getAttribute('Site')) {
            return $this->getAttribute('Site');
        }

        return QUI::getRewrite()->getSite();
    }
}
