<?php

namespace QUI\Locations\Handler;

use QUI\Utils\System\File as QUIFile;
use QUI\Utils\Text\XML;
use QUI\Utils\XML\Settings as XMLSettings;
use QUI;
use QUI\Cache\Manager as CacheManager;
use QUI\Locations\Utils;

/**
 * Handles location attributes
 *
 * @package QUI\Locations
 */
class Attributes
{
    const TBL_LOCATIONS_ATTRIBUTES = 'quiqqer_locations_attributes';

    /**
     * Attributes runtime cache
     *
     * @var array
     */
    protected static $attributes = [];

    /**
     * Get a single location Attribute
     *
     * @param string $name - unique location attribute name
     * @return QUI\Locations\Attributes\AbstractAttribute
     */
    public static function getLocationAttribute($name)
    {
        if (isset(self::$attributes[$name])) {
            return self::$attributes[$name];
        }

        $attributeClass          = self::getLocationAttributeClass($name);
        self::$attributes[$name] = new $attributeClass($name);

        return self::$attributes[$name];
    }

    /**
     * Get the attribute class for a specific attribute
     *
     * @param string $name - unique location attribute name
     * @return string - class name with fully qualified namespace
     *
     * @throws QUI\Locations\Exception
     */
    public static function getLocationAttributeClass($name)
    {
        $cacheName = 'quiqqer/locations/attributes/class/'.$name;

        try {
            return CacheManager::get($cacheName);
        } catch (\Exception $Exception) {
            // if not cached, try to determine attribute class
        }

        $result = QUI::getDataBase()->fetch([
            'select' => 'attribute_class',
            'from'   => self::getLocationAttributesTable(),
            'where'  => [
                'name' => $name
            ]
        ]);

        if (empty($result)) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.attribute.not.found',
                [
                    'name' => $name
                ]
            ], 404);
        }

        $class = $result[0]['attribute_class'];

        if (is_null($class)) {
            $attributeClass = '\\QUI\\Locations\\Attributes\\Types\\Input';
        } else {
            $attributeClass = $class;
        }

        if (!class_exists($attributeClass)) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.attributes.attribute.class.not.found',
                [
                    'name'  => $name,
                    'class' => $class
                ]
            ], 404);
        }

        CacheManager::set($cacheName, $attributeClass);

        return $attributeClass;
    }

    /**
     * Get list of all attributes (name, title, description)
     *
     * @return array
     */
    public static function getList()
    {
        $result = QUI::getDataBase()->fetch([
            'select' => [
                'name',
                'title',
                'description'
            ],
            'from'   => self::getLocationAttributesTable(),
            'order'  => 'priority DESC'
        ]);

        $list = [];

        foreach ($result as $row) {
            $title       = Utils::getLocaleFromArray(json_decode($row['title'], true));
            $description = json_decode($row['description'], true);

            if (!empty($description)) {
                $description = Utils::getLocaleFromArray($description);
            }

            $list[] = [
                'name'        => $row['name'],
                'title'       => $title,
                'description' => $description
            ];
        }

        return $list;
    }

    /**
     * Get list of columns with caption that are shown in location overview list
     *
     * @return array
     */
    public static function getLocationListColumns()
    {
        $attributeIds = QUI::getPackage('quiqqer/locations')->getConfig()->get('settings', 'listColumns');

        if (empty($attributeIds)) {
            $attributeIds = ['name', 'description'];
        } else {
            $attributeIds = explode(',', $attributeIds);
        }

        $result = QUI::getDataBase()->fetch([
            'select' => [
                'name',
                'title'
            ],
            'from'   => self::getLocationAttributesTable(),
            'where'  => [
                'name' => [
                    'type'  => 'IN',
                    'value' => $attributeIds
                ]
            ],
            'order'  => 'priority DESC'
        ]);

        $columns = [];

        foreach ($result as $row) {
            $columns[] = [
                'name'   => $row['name'],
                'header' => Utils::getLocaleFromArray(json_decode($row['title'], true))
            ];
        }

        return $columns;
    }

    /**
     * Get template for a specific template category
     *
     * @param string $category - category name
     * @return string
     *
     * @throws QUI\Locations\Exception
     */
    public static function getAttributeCategoryTemplate($category)
    {
        $cacheName = 'quiqqer/locations/attributes/categories/'.$category;

        try {
            return CacheManager::get($cacheName);
        } catch (\Exception $Exception) {
            // read category template from location.xml file
        }

        $xmlFiles = self::getLocationXmlFiles();

        $XMLSettingsParser = XMLSettings::getInstance();
        $XMLSettingsParser->setXMLPath('//quiqqer/locations/window');

        $template = $XMLSettingsParser->getCategoriesHtml($xmlFiles, $category);

        if ($template === false) {
            throw new QUI\Locations\Exception([
                'quiqqer/locations',
                'exception.handler.attributes.category.not.found',
                [
                    'category' => $category
                ]
            ], 404);
        }

        CacheManager::set($cacheName, $template);

        return $template;
    }

    /**
     * Get attribut categories from locations.xml files from all modules
     *
     * @return array
     */
    public static function getAttributeCategories()
    {
        $cacheName = 'quiqqer/locations/attributes/categories/all';

        try {
            $result = CacheManager::get($cacheName);
        } catch (\Exception $Exception) {
            // read categories from location.xml files
            $xmlFiles          = self::getLocationXmlFiles();
            $XMLSettingsParser = XMLSettings::getInstance();
            $XMLSettingsParser->setXMLPath('//quiqqer/locations/window');

            $CategoryCollection = $XMLSettingsParser->getCategories($xmlFiles)->sort(function ($a, $b) {
                return $a['index'] > $b['index'];
            });

            $result = array_values($CategoryCollection->toArray());

            CacheManager::set($cacheName, $result);
        }

        // category title translation
        foreach ($result as $key => $category) {
            if (is_array($category['title'])) {
                $title = QUI::getLocale()->get(
                    $category['title'][0],
                    $category['title'][1]
                );

                $category['text']  = $title;
                $category['title'] = $title;
                $category['alt']   = $title;
            }

            $result[$key] = $category;
        }

        return $result;
    }

    /**
     * Get locations.xml files from all installed plugins
     *
     * @return array - fq filepath
     */
    protected static function getLocationXmlFiles()
    {
        $packages = QUIFile::readDir(OPT_DIR);
        $xmlFiles = [];

        // read all locations.xml files and parse locale attributes
        foreach ($packages as $package) {
            if ($package === 'composer') {
                continue;
            }

            $packageDir = OPT_DIR.'/'.$package;
            $list       = QUIFile::readDir($packageDir);

            foreach ($list as $sub) {
                if (!is_dir($packageDir.'/'.$sub)) {
                    continue;
                }

                $xmlFile = $packageDir.'/'.$sub.'/locations.xml';

                if (!file_exists($xmlFile)) {
                    continue;
                }

                $xmlFiles[] = $xmlFile;
            }
        }

        return $xmlFiles;
    }

    /**
     * Get table name for location attributes table
     *
     * @return string
     */
    public static function getLocationAttributesTable()
    {
        return QUI::getDBTableName(self::TBL_LOCATIONS_ATTRIBUTES);
    }
}
