<?php

namespace QUI\Locations\Handler;

use QUI;
use QUI\Geo\Utils\Geo;
use QUI\LocationCategories\Handler as CategoryHandler;
use QUI\Utils\Security\Orthos;
use QUI\Locations\Locator;

/**
 * Class LocationSearch
 *
 * Manager for LocationSearch purposes
 */
class LocationSearch
{
    const SUBMIT_ACTION_LIST = 'list';
    const SUBMIT_ACTION_MAP  = 'map';

    const SITE_TYPE_LOCATIONSEARCH                  = 'quiqqer/locations:types/locationsearch';
    const SITE_ATTR_RESULTS_PER_PAGE                = 'quiqqer.locations.locationsearch.results_per_page';
    const SITE_ATTR_GENERAL_RESULTS_ON_EMPTY_SEARCH = 'quiqqer.locations.locationsearch.general_results_on_empty_search';
    const SITE_ATTR_COUNTRY                         = 'quiqqer.locations.locationsearch.country';
    const SITE_ATTR_RADIUS                          = 'quiqqer.locations.locationsearch.radius';

    /**
     * Get LocationSearch site
     *
     * @param QUI\Projects\Project $Project (optional)
     * @return QUI\Projects\Site|false
     */
    public static function getLocationSearchSite(QUI\Projects\Project $Project = null)
    {
        if (empty($Project)) {
            try {
                $Project = QUI::getRewrite()->getProject();
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
                return false;
            }
        }

        $sites = $Project->getSites([
            'where' => [
                'type' => self::SITE_TYPE_LOCATIONSEARCH
            ],
            'limit' => 1
        ]);

        if (empty($sites)) {
            return false;
        }

        return $sites[0];
    }

    /**
     * Get GeoCenter based on the current search request
     *
     * @return QUI\Geo\Utils\GeoLocation|false - GeoLocation or false if not determinable
     */
    public static function getGeoCenterByRequest()
    {
        if (empty($_REQUEST['zip'])) {
            return false;
        }

        try {
            $settings = QUI::getPackage('quiqqer/locations')->getConfig()->getSection('locationsearch');
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }

        try {
            $GeoApi       = Geo::getApi();
            $geoLocations = $GeoApi->getGeoLocationsByZip(
                Orthos::clear($_REQUEST['zip']),
                $settings['country']
            );
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }

        if (empty($geoLocations)) {
            return false;
        }

        return \current($geoLocations);
    }

    /**
     * Get LocationSearch search radius
     *
     * @param QUI\Projects\Site $Site (optional)
     * @return int
     */
    public static function getSearchRadius(QUI\Projects\Site $Site = null)
    {
        if (empty($Site)) {
            $Site = self::getLocationSearchSite();
        }

        try {
            $radius = (int)$Site->getAttribute('quiqqer.locations.locationsearch.radius');
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            $radius = 100;
        }

        return $radius;
    }

    /**
     * Get all location IDs parsed from the search request
     *
     * @return int[]
     */
    public static function getLocationIdsByRequest()
    {
        $locationIds = [];

        if (!empty($_REQUEST['geo_identifier'])) {
            try {
                $GeoLocation = Geo::getApi()->getGeoLocationByGeoSuggestionIdentifier(
                    Orthos::clear($_REQUEST['geo_identifier'])
                );

                $locations = Locator::getLocationsInRadius(
                    $GeoLocation->getPosition(),
                    [],
                    self::getSearchRadius()
                );

                foreach ($locations as $location) {
                    $locationIds[] = $location['location']['id'];
                }
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
//                $locationIds = Locations::getAllLocationIds(false);
            }
        } else {
//            $locationIds = Locations::getAllLocationIds(false);
        }

        // Filter locations by category?
        $restrictToLocationIds = false;

        if (!empty($_REQUEST['cat_id']) && \is_numeric($_REQUEST['cat_id'])) {
            $categoryId = (int)$_REQUEST['cat_id'];

            if (QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
                try {
                    $restrictToLocationIds = CategoryHandler::getCategory($categoryId)->getLocationIds(false);
                } catch (\Exception $Exception) {
                    QUI\System\Log::writeException($Exception);
                }
            }
        }

        if ($restrictToLocationIds !== false) {
            $locationIds = \array_intersect($locationIds, $restrictToLocationIds);
        }

        return $locationIds;
    }
}
