<?php

namespace QUI\Locations\Handler;

use QUI;
use QUI\Locations\Handler\Attributes as AttributesHandler;
use QUI\Locations\Handler\Locations as LocationsHandler;
use QUI\Utils\Grid;
use QUI\Utils\Security\Orthos;
use QUI\Locations\Location\Location;
use QUI\Permissions\Permission;
use QUI\Projects\Media\Utils as MediaUtils;
use function GuzzleHttp\Promise\queue;

/**
 * Handles multiple locations
 *
 * @package QUI\Locations
 */
class Locations
{
    const TBL_LOCATIONS = 'quiqqer_locations';
    const TBL_LOCATIONS_SEARCH = 'quiqqer_locations_search';
    const PERM_FORCE_EDIT_LOCATION = 'quiqqer.locations.force_edit_location';

    /**
     * Locations runtime cache
     *
     * @var array
     */
    protected static $locations = [];

    /**
     * The Locale (language) that is used in the quiqqer/locations context
     *
     * @var QUI\Locale
     */
    protected static $LocationsLocale = null;

    /**
     * Get the Locale (language) that is used in the quiqqer/locations context
     *
     * @return QUI\Locale
     */
    public static function getLocale()
    {
        if (is_null(self::$LocationsLocale)) {
            return QUI::getLocale();
        }

        return self::$LocationsLocale;
    }

    /**
     * Set the language that is used in the quiqqer/locations context
     *
     * @param QUI\Locale $Locale
     */
    public static function setLocale($Locale)
    {
        self::$LocationsLocale = $Locale;
    }

    /**
     * Create a new location
     *
     * @param string $name - Location name
     * @param QUI\Users\User|QUI\Users\SystemUser $User (optional) - Parent User
     * @return int - new location ID
     * @throws QUI\Database\Exception
     * @throws QUI\Locations\Exception
     * @throws QUI\Exception
     * @throws \QUI\Permissions\Exception
     */
    public static function createLocation($name, $User = null)
    {
        if (empty($User)) {
            $User = QUI::getUserBySession();
        }

        Permission::checkPermission(Location::PERMISSION_CREATE, $User);

        $NameAttribute = AttributesHandler::getLocationAttribute('name');
        $NameAttribute->setValue($name);

        // insert in locations table
        QUI::getDataBase()->insert(
            self::getLocationsTable(),
            [
                'attributes' => json_encode([
                    'name' => $NameAttribute->getValue()
                ]),
                'c_user'     => $User->getId(),
                'c_date'     => time(),
                'e_user'     => $User->getId(),
                'e_date'     => time()
            ]
        );

        $newLocationId = QUI::getDataBase()->getPDO()->lastInsertId();

        // insert locations search table
        $langs          = QUI::availableLanguages();
        $LocationLocale = new QUI\Locale();

        foreach ($langs as $lang) {
            $LocationLocale->setCurrent($lang);

            QUI::getDataBase()->insert(
                self::getLocationsSearchTable(),
                [
                    'id'   => $newLocationId,
                    'name' => $NameAttribute->getSearchCacheValue($LocationLocale),
                    'lang' => $lang
                ]
            );
        }

        $NewLocation = self::getLocation($newLocationId);

        QUI::getEvents()->fireEvent('onLocationCreate', [$NewLocation]);

        return $newLocationId;
    }

    /**
     * Get a specific location
     *
     * @param $id
     * @return Location
     * @throws QUI\Locations\Exception
     */
    public static function getLocation($id)
    {
        if (isset(self::$locations[$id])) {
            return self::$locations[$id];
        }

        self::$locations[$id] = new Location($id);

        return self::$locations[$id];
    }

    /**
     * Get IDs of all locations
     *
     * @param bool $includeInactive (optional) - Include inactive locations [default: true]
     * @return int[]
     */
    public static function getAllLocationIds($includeInactive = true)
    {
        $where = [];

        if (!$includeInactive) {
            $where['active'] = 1;
        }

        try {
            $result = QUI::getDataBase()->fetch([
                'select' => [
                    'id'
                ],
                'from'   => self::getLocationsTable(),
                'where'  => $where
            ]);

            return \array_column($result, 'id');
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return [];
        }
    }

    /**
     * Get all locations
     *
     * @param bool $includeInactive (optional) - Include inactive locations [default: true]
     * @return Location[]
     */
    public static function getAllLocations($includeInactive = true)
    {
        $locations = [];

        foreach (self::getAllLocationIds($includeInactive) as $locationId) {
            try {
                $locations[] = self::getLocation($locationId);
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        return $locations;
    }

    /**
     * Get IDs of all locations with a specific attribute value
     *
     * @param string $attribute
     * @param string $value
     * @param bool $includeInactive (optional) - Include inactive locations [default: true]
     * @return int[]
     * @throws QUI\Database\Exception
     */
    public static function getLocationIdsWithAttributeValue($attribute, $value, $includeInactive = true)
    {
        // check if attribute exists
        AttributesHandler::getLocationAttribute($attribute);

        $where = [
            $attribute => $value
        ];

        if (!$includeInactive) {
            $where['active'] = 1;
        }

        $result = QUI::getDataBase()->fetch([
            'select' => [
                'id'
            ],
            'from'   => self::getLocationsSearchTable(),
            'where'  => $where
        ]);

        return \array_values(\array_unique(\array_column($result, 'id')));
    }

    /**
     * Get all locations with a specific attribute value
     *
     * @param string $attribute - location attribute name
     * @param string $value - the value of the attribute
     * @param bool $includeInactive (optional) - Include inactive locations [default: true]
     * @return Location[]
     *
     * @throws QUI\Exception
     */
    public static function getLocationsWithAttributeValue($attribute, $value, $includeInactive = true)
    {
        $locationIds = self::getLocationIdsWithAttributeValue($attribute, $value, $includeInactive);
        $locations   = [];

        foreach ($locationIds as $locationId) {
            $locations[] = self::getLocation($locationId);
        }

        return $locations;
    }

    /**
     * Search locations
     *
     * @param array $searchParams - search parameters
     * @param bool $countOnly (optional) - get result count only [default: false]
     * @return array|int - Location data as array or count as int
     */
    public static function search($searchParams, bool $countOnly = false)
    {
        $PDO   = QUI::getDataBase()->getPDO();
        $binds = [];
        $where = [];

        $Grid       = new Grid($searchParams);
        $gridParams = $Grid->parseDBParams($searchParams);

        if ($countOnly) {
            $sql = "SELECT COUNT(*)";
        } else {
            $sql = "SELECT *";
        }

        $sql .= " FROM ".self::getLocationsSearchTable();

        if (!empty($searchParams['searchterm'])) {
            $whereOR = [];

            $attributes = AttributesHandler::getList();

            foreach ($attributes as $attribute) {
                $column = $attribute['name'];

                $whereOR[]      = '`'.$column.'` LIKE :'.$column;
                $binds[$column] = [
                    'value' => '%'.$searchParams['searchterm'].'%',
                    'type'  => \PDO::PARAM_STR
                ];
            }

            $where[] = '('.implode(' OR ', $whereOR).')';
        }

        $where[]       = 'lang = :lang';
        $binds['lang'] = [
            'value' => Locations::getLocale()->getCurrent(),
            'type'  => \PDO::PARAM_STR
        ];

        // build WHERE query string
        if (!empty($where)) {
            $sql .= " WHERE ".implode(" AND ", $where);
        }

        if (!empty($searchParams['sortOn'])) {
            $order = "ORDER BY ".Orthos::clear($searchParams['sortOn']);

            if (!empty($searchParams['sortBy'])) {
                $order .= " ".Orthos::clear($searchParams['sortBy']);
            } else {
                $order .= " ASC";
            }

            $sql .= " ".$order;
        }

        if (!empty($gridParams['limit']) && !$countOnly) {
            $sql .= " LIMIT ".$gridParams['limit'];
        } else {
            if (!$countOnly) {
                $sql .= " LIMIT ".(int)20;
            }
        }

        $Stmt = $PDO->prepare($sql);

        // bind search values
        foreach ($binds as $var => $bind) {
            $Stmt->bindValue(':'.$var, $bind['value'], $bind['type']);
        }

        // fetch information for all corresponding passwords
        try {
            $Stmt->execute();
            $result = $Stmt->fetchAll(\PDO::FETCH_ASSOC);
        } catch (\Exception $Exception) {
            QUI\System\Log::addError(
                self::class.' :: search() -> '.$Exception->getMessage()
            );

            return [];
        }

        if ($countOnly) {
            return (int)current(current($result));
        }

        return $result;
//
//        foreach ($result as $row) {
//            $locationIds[] = $row['id'];
//        }
//
//        return $locationIds;
    }

    /**
     * Get table name of location table
     *
     * @return string
     */
    public static function getLocationsTable()
    {
        return QUI::getDBTableName(self::TBL_LOCATIONS);
    }

    /**
     * Get table name of location search table
     *
     * @return string
     */
    public static function getLocationsSearchTable()
    {
        return QUI::getDBTableName(self::TBL_LOCATIONS_SEARCH);
    }

    /**
     * Get main media folder for locations
     *
     * @return false|QUI\Projects\Media\Folder
     *
     * @throws \Exception
     */
    public static function getMainMediaFolder()
    {
        $Config          = QUI::getPackage('quiqqer/locations')->getConfig();
        $mainMediaFolder = $Config->get('settings', 'mainMediaFolder');

        if (empty($mainMediaFolder)) {
            return false;
        }

        try {
            $Item = MediaUtils::getMediaItemByUrl($mainMediaFolder);

            if (MediaUtils::isFolder($Item)) {
                /** @var QUI\Projects\Media\Folder $Item */
                return $Item;
            }
        } catch (\Exception $Exception) {
            if ($Exception->getCode() == 404) {
                return false;
            }

            throw $Exception;
        }

        return false;
    }

    /**
     * Get location default image
     *
     * @return QUI\Projects\Media\Image|false - Image or false if none set
     */
    public static function getLocationDefaultImage()
    {
        try {
            $Config = QUI::getPackage('quiqqer/locations')->getConfig();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }

        $defaultImage = $Config->get('settings', 'defaultLocationImage');

        if (empty($defaultImage)) {
            return false;
        }

        try {
            $Item = MediaUtils::getMediaItemByUrl($defaultImage);

            if (MediaUtils::isImage($Item)) {
                /** @var QUI\Projects\Media\Image $Item */
                return $Item;
            }
        } catch (\Exception $Exception) {
            if ($Exception->getCode() != 404) {
                QUI\System\Log::writeException($Exception);
            }
        }

        return false;
    }

    /**
     * Get a location by URL
     *
     * @param string $url
     * @return QUI\Locations\Location\Location|false - Location or false of no location found
     */
    public static function getLocationByUrl($url)
    {
        try {
            $result = QUI::getDataBase()->fetch([
                'select' => ['id'],
                'from'   => LocationsHandler::getLocationsTable(),
                'where'  => [
                    'url_slugs' => [
                        'type'  => '%LIKE%',
                        'value' => ','.$url.','
                    ]
                ],
                'limit'  => 1
            ]);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }

        if (empty($result)) {
            return false;
        }

        try {
            return LocationsHandler::getLocation((int)$result[0]['id']);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return false;
        }
    }
}
