<?php

namespace QUI\Locations;

use QUI;
use QUI\Cache\Manager as QUICacheManager;
use QUI\Locations\Handler\Attributes as AttributesHandler;
use QUI\Locations\Location\Location;
use QUI\LocationCategories\Handler as LocationCategories;
use QUI\Locations\Handler\Locations;

/**
 * Class LocationEvents
 *
 * quiqqer/locations Location Events
 *
 * @package QUI\Locations
 */
class LocationEvents
{
    /**
     * Location Event: onLocationDelete
     *
     * @param Location $Location - The location that is to be deleted
     * @return void
     */
    public static function onLocationDelete($Location)
    {
        $attributes = AttributesHandler::getList();

        foreach ($attributes as $attribute) {
            $Attribute = AttributesHandler::getLocationAttribute($attribute['name']);

            if (method_exists($Attribute, 'onLocationDelete')) {
                $Attribute->onLocationDelete($Location);
            }
        }
    }

    /**
     * quiqqer/locations: onLocationSaveBefore
     *
     * @param Location $Location
     * @return void
     */
    public static function onLocationSaveBefore(Location $Location)
    {
//        self::setLocationUrlSlug($Location);
    }

    /**
     * quiqqer/locations: onLocationActivate
     *
     * @param Location $Location
     * @return void
     */
    public static function onLocationActivate(Location $Location)
    {
        self::clearLocationCache($Location);
        self::setLocationUrlSlug($Location);
    }

    /**
     * quiqqer/locations: onLocationSave
     *
     * @param Location $Location
     * @return void
     */
    public static function onLocationSave(Location $Location)
    {
        self::clearLocationCache($Location);
        self::setLocationUrlSlug($Location);
    }

    /**
     * Automatically set Location URL slug based on title (if not set!)
     *
     * @param Location $Location
     * @return void
     */
    protected static function setLocationUrlSlug(Location $Location)
    {
        try {
            $UrlSlugAttribute = $Location->getLocationAttribute('url_slug');
            $NameAttribute    = $Location->getLocationAttribute('name');
            $urlSlugValue     = $UrlSlugAttribute->getValue();
            $nameValue        = $NameAttribute->getValue();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return;
        }

        if (empty($nameValue)) {
            return;
        }

        $nameValue = \json_decode($nameValue, true);
        $langs     = QUI::availableLanguages();
        $checkHash = false;

        if (empty($urlSlugValue)) {
            $urlSlugValue = [];

            foreach ($langs as $lang) {
                if (!empty($nameValue[$lang])) {
                    $urlSlugValue[$lang] = $nameValue[$lang];
                }
            }
        } else {
            $checkHash    = \md5($urlSlugValue);
            $urlSlugValue = \json_decode($urlSlugValue, true);

            foreach ($langs as $lang) {
                if (empty($urlSlugValue[$lang] && !empty($nameValue[$lang]))) {
                    $urlSlugValue[$lang] = $nameValue[$lang];
                }
            }
        }

        // Check if URL slugs are valid
        foreach ($urlSlugValue as $lang => $urlSlug) {
            if (empty($urlSlug)) {
                continue;
            }

            $where = [
                'lang'     => $lang,
                'url_slug' => $urlSlug
            ];

            if (QUI::getPackageManager()->isInstalled('quiqqer/locationcategories')) {
                $Category = LocationCategories::getCategoryByLocation($Location);

                if ($Category) {
                    $where['id'] = [
                        'type'  => 'IN',
                        'value' => $Category->getLocationIds(true, false, true)
                    ];
                }
            }

            $result = QUI::getDataBase()->fetch([
                'select' => ['id'],
                'from'   => Locations::getLocationsSearchTable(),
                'where'  => $where
            ]);

            $count = 0;

            foreach ($result as $row) {
                if ((int)$row['id'] === $Location->getId()) {
                    continue;
                }

                $count++;
            }

            if ($count && !empty($urlSlugValue[$lang])) {
                $urlSlugValue[$lang] = $urlSlug.'-'.$count;
            }
        }

        try {
            $urlSlugValue = \json_encode($urlSlugValue);

            if (\md5($urlSlugValue) !== $checkHash) {
                $UrlSlugAttribute->setValue($urlSlugValue);
                $Location->save(false);
            }
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }

    /**
     * Clear relevant cache entries for a location
     *
     * @param Location $Location
     * @return void
     */
    protected static function clearLocationCache(Location $Location)
    {
        QUICacheManager::clear('quiqqer/locations/'.$Location->getId());
    }
}
