<?php

namespace QUI\Locations;

use QUI;
use QUI\Geo\Utils\GeoPoint;
use QUI\Locations\Location\Location;
use QUI\Locations\Handler\Locations;
use QUI\Geo\Utils\Geo;
use QUI\Cache\Manager as QUICacheManager;

/**
 * Class Locator
 *
 * Handles location searches for the Frontend
 *
 * Search locations based on different criteria
 */
class Locator
{
    /**
     * Get a list of locations that are in a radius of a given center point
     *
     * @param GeoPoint|false $Center - Geographic center; if set to false ignore radius and return all available locations
     * @param array $searchParams - Search parameters (page, perPage etc.)
     * @param float (optional) - Maximum radius in km (kilometers); must be positive [default: 20.0 km]
     * @return array|int - Location list (location + distance) or count
     * @throws \QUI\Geo\Utils\Exception
     *
     * @thanks https://stackoverflow.com/a/23118314
     */
    public static function getLocationsInRadius($Center, $searchParams, float $radius = 20.0)
    {
        $GeoApi = Geo::getApi();

        // Build query
        $sql   = "SELECT `id`, `lat`, `lng` FROM ".Locations::getLocationsSearchTable();
        $where = [
            '`lang` = "'.Locations::getLocale()->getCurrent().'"',
            '`active` = 1'
        ];

        // filter
        if (isset($searchParams['allowedIds']) && \is_array($searchParams['allowedIds'])) {
            if (empty($searchParams['allowedIds'])) {
                if (!empty($searchParams['count'])) {
                    return 0;
                }

                return [];
            }

            $where[] = '`id` IN('.implode(',', $searchParams['allowedIds']).')';
        }

        if ($Center) {
            $centerLng = $Center->getLongitude();
            $centerLat = $Center->getLatitude();

            // Calculate border box
            $kmLngDegree = 111.320 * cos($centerLng / 180.0 * pi());
            $deltaLat    = $radius / 111.1;
            $deltaLng    = $radius / $kmLngDegree;

            $minLat = $centerLat - $deltaLat;
            $maxLat = $centerLat + $deltaLat;

            $minLng = $centerLng - $deltaLng;
            $maxLng = $centerLng + $deltaLng;

            $where[] = '`lat` >= '.$minLat;
            $where[] = '`lat` <= '.$maxLat;
            $where[] = '`lng` >= '.$minLng;
            $where[] = '`lng` <= '.$maxLng;
        }

        $sql .= " WHERE ".implode(" AND ", $where);

        $locations = [];

        try {
            $results = QUI::getDataBase()->fetchSQL($sql);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return $locations;
        }

        if (!empty($searchParams['count'])) {
            return count($results);
        }

        // Sort by distance
        if ($Center) {
            usort($results, function ($a, $b) use ($Center, $GeoApi) {
                $distanceA = $GeoApi->getLinearDistance($Center, new GeoPoint((float)$a['lat'], (float)$a['lng']));
                $distanceB = $GeoApi->getLinearDistance($Center, new GeoPoint((float)$b['lat'], (float)$b['lng']));

                if ($distanceA === $distanceB) {
                    return 0;
                }

                return $distanceA < $distanceB ? -1 : 1;
            });
        }

        // LIMIT
        if (empty($searchParams['noLimit'])) {
            $page = 1;

            if (!empty($searchParams['page'])) {
                $page = (int)$searchParams['page'];
            }

            $perPage = 10;

            if (!empty($searchParams['perPage'])) {
                $perPage = $searchParams['perPage'];
            }

            $results = array_slice($results, ($page - 1) * $perPage, $perPage);
        }

        foreach ($results as $row) {
            try {
                $location = Locations::getLocation($row['id'])->getView()->toArray();
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
                continue;
            }

            // @todo maybe use route distance? depending on setting; could take long processing time
            $distance = false;

            if ($Center) {
                $distance = $GeoApi->getLinearDistance($Center, new GeoPoint((float)$row['lat'], (float)$row['lng']));
                $distance /= 1000;
                $distance = round($distance, 1);
            }

            $locations[] = [
                'location' => $location,
                'distance' => $distance
            ];
        }

        return $locations;
    }

    /**
     * Get coordinates of active locations (if coordinates are set!)
     *
     * @param array $searchParams (optional) - Search parameters (e.g. filters)
     * @return array
     * @throws QUI\Database\Exception
     */
    public static function getAllLocationPoints($searchParams = [])
    {
        $where = [
            'active'      => 1,
            'lang'        => Locations::getLocale()->getCurrent(),
            'coordinates' => [
                'type'  => 'NOT',
                'value' => null
            ]
        ];

        if (isset($searchParams['allowedIds']) && \is_array($searchParams['allowedIds'])) {
            if (empty($searchParams['allowedIds'])) {
                return [];
            }

            $where['id'] = [
                'type'  => 'IN',
                'value' => $searchParams['allowedIds']
            ];
        }

        $result = QUI::getDataBase()->fetch([
            'select' => ['id', 'lat', 'lng'],
            'from'   => Locations::getLocationsSearchTable(),
            'where'  => $where
        ]);

        // Sort
        array_multisort(
            array_column($result, 'lat'),
            \SORT_ASC,
            array_column($result, 'lng'),
            \SORT_ASC,
            $result
        );

        foreach ($result as $k => $row) {
            $result[$k]['map_icon'] = self::getLocationMapIcon($row['id']);
        }

        return $result;
    }

    /**
     * Get map icon for location
     *
     * @param int $locationId
     * @return string|false - URL to Icon or false if no icon set
     */
    public static function getLocationMapIcon(int $locationId)
    {
        $cacheName = 'quiqqer/locations/'.$locationId.'/map_icon';

        try {
            return QUICacheManager::get($cacheName);
        } catch (\Exception $Exception) {
            // nothing, re-build cache entry
        }

        try {
            $Location = Locations::getLocation($locationId);
            $MapIcon  = $Location->getMapIcon();

            if (!empty($MapIcon)) {
                $mapIconUrl = $MapIcon->getSizeCacheUrl(30, 30); // @todo custom dimensions?
                QUICacheManager::set($cacheName, $mapIconUrl);

                return $mapIconUrl;
            }
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }

        return false;
    }
}
