<?php

namespace QUI\Locations\Locator;

use QUI;
use QUI\Locations\Locator\Filter\LocatorFilterInterface;
use function GuzzleHttp\Psr7\_parse_request_uri;

/**
 * Class Handler
 *
 * Main handler for Locator operations
 */
class Handler
{
    const SITE_TYPE_LOCATOR = 'quiqqer/locations:types/locator';

    const MAP_EMPTY_RESULTS_SHOW_ALL  = 'all';
    const MAP_EMPTY_RESULTS_SHOW_NONE = 'none';

    /**
     * Get list of allowed Location IDs based on Locator Filters
     *
     * @param array $filters - Filter data from frontend Filter controls
     * @return int[]|false - List of IDs or false if no ID restrictions apply
     */
    public static function getAllowedLocationIds(array $filters)
    {
        $filterProviders = self::getFilterProviders();
        $allowedIds      = [];

        foreach ($filters as $filterProviderHash => $filterData) {
            if (empty($filterData)) {
                continue;
            }

            if (!isset($filterProviders[$filterProviderHash])) {
                continue;
            }

            /** @var LocatorFilterInterface $FilterProvider */
            $Filter             = $filterProviders[$filterProviderHash];
            $allowedIdsByFilter = $Filter->getAllowedLocationIds($filterData);

            if ($allowedIdsByFilter !== false) {
                $allowedIds[] = $allowedIdsByFilter;
            }
        }

        if (empty($allowedIds)) {
            return false;
        }

        // cleanup
        if (\count($allowedIds) > 1) {
            $allowedIds = \array_values(
                \array_unique(
                    \call_user_func_array('array_intersect', $allowedIds)
                )
            );
        } else {
            $allowedIds = \current($allowedIds);
        }

        return $allowedIds;
    }

    /**
     * @return LocatorFilterInterface[]
     * @todo caching
     *
     * Get all installed Locator Filter providers
     *
     */
    public static function getFilterProviders()
    {
        $providers = [];
        $installed = QUI::getPackageManager()->getInstalled();

        foreach ($installed as $package) {
            try {
                $Package = QUI::getPackage($package['name']);

                if (!$Package->isQuiqqerPackage()) {
                    continue;
                }

                $importProviderClasses = $Package->getProvider('locator-filter');

                foreach ($importProviderClasses as $class) {
                    if (\is_a($class, LocatorFilterInterface::class, true)) {
                        $idHash             = \mb_substr(\hash('sha256', $class), 0, 5);
                        $providers[$idHash] = new $class();
                    }
                }
            } catch (QUI\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        return $providers;
    }

    /**
     * Get all Filter controls.
     *
     * @return QUI\Locations\Locator\Filter\AbstractFilterControl[]
     */
    public static function getFilterControls()
    {
        $filterControls = [];

        foreach (self::getFilterProviders() as $hash => $Filter) {
            $filterControls[$hash] = $Filter->getControl();
        }

        return $filterControls;
    }

    /**
     * Get Locator site for $Project
     *
     * @param QUI\Projects\Project $Project (optional) - [default: get Project via Rewrite]
     * @return QUI\Projects\Site|false - QUIQQER Site object or false if Site does not exist
     */
    public static function getLocatorSite(QUI\Projects\Project $Project = null)
    {
        if (empty($Project)) {
            try {
                $Project = QUI::getRewrite()->getProject();
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
                return false;
            }
        }

        $sites = $Project->getSites([
            'where' => [
                'type' => Handler::SITE_TYPE_LOCATOR
            ],
            'limit' => 1
        ]);

        if (empty($sites)) {
            return false;
        }

        return $sites[0];
    }

    /**
     * Get Locator setting
     *
     * @param string $key
     * @return string|int
     *
     * @throws QUI\Exception
     */
    public static function getLocatorSetting($key)
    {
        $Conf = QUI::getPackage('quiqqer/locations')->getConfig();
        return $Conf->get('locator', $key);
    }

    /**
     * Get LocatorSearch setting
     *
     * @param string $key
     * @return string|int
     *
     * @throws QUI\Exception
     */
    public static function getLocatorSearchSetting($key)
    {
        $Conf = QUI::getPackage('quiqqer/locations')->getConfig();
        return $Conf->get('locationsearch', $key);
    }

    /**
     * Get default view center for Locator map
     *
     * @param QUI\Projects\Site $Site (optional) - Get view center from Site object
     * @return array
     */
    public static function getDefaultMapViewCenter(QUI\Projects\Site $Site = null)
    {
        $fallbackViewCenter = [
            'lat'  => 51.23835708678009,
            'lng'  => 9.69726409912111,
            'zoom' => 6
        ];

        if ($Site) {
            $viewCenter = $Site->getAttribute('quiqqer.locations.locator.map.center');

            if (!empty($viewCenter)) {
                return \json_decode($viewCenter, true);
            }
        }

        try {
            $defaultMapViewCenter = self::getLocatorSetting('mapDefaultCenter');

            if (empty($defaultMapViewCenter)) {
                return $fallbackViewCenter;
            }

            return \json_decode($defaultMapViewCenter, true);
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return $fallbackViewCenter;
        }
    }
}
