<?php

namespace QUI\Locations;

use QUI;
use QUI\Utils\Security\Orthos;

/**
 * General utilities for quiqqer/locations
 *
 * @package QUI\Locations
 */
class Utils
{
    /**
     * Clear a JSON string from evil code but keep JSON intact
     *
     * @param string $str - JSON string
     * @return string - cleared JSON string
     */
    public static function clearJSONString($str)
    {
        $str = Orthos::removeHTML($str);
//        $str = Orthos::clearPath($str);
//        $str = Orthos::clearFormRequest($str);

        $str = \str_replace(['../', '..'], '', $str);   // custom clearPath
        $str = htmlspecialchars($str, ENT_NOQUOTES);

        return $str;
    }

    /**
     * Clear an array that contains JSON-strings
     *
     * @param array $array
     * @return array - cleared array
     */
    public static function clearArrayWithJSON(array $array)
    {
        if (!is_array($array)) {
            return [];
        }

        foreach ($array as $k => $v) {
            if (is_array($v)) {
                $array[$k] = self::clearArrayWithJSON($v);
                continue;
            }

            $array[$k] = self::clearJSONString($v);
        }

        return $array;
    }

    /**
     * Get translation from locale data array
     *
     * @param array $localeData
     * @return string
     */
    public static function getLocaleFromArray($localeData)
    {
        return QUI::getLocale()->get($localeData['group'], $localeData['var']);
    }

    /**
     * Checks if given the current time is in a given time window
     *
     * @param array $timeData - time window data
     * @return bool
     */
    public static function isOpen($timeData)
    {
        $openDay     = false;
        $openTime    = false;
        $currentDay  = mb_substr(mb_strtolower(date("D")), 0, 2);
        $currentTime = date("Hi");

        switch ($timeData['range']) {
            case 'motofr':
                if ($currentDay != 'sa'
                    && $currentDay != 'su'
                ) {
                    $openDay = true;
                }
                break;

            case 'motosa':
                if ($currentDay != 'su') {
                    $openDay = true;
                }
                break;

            case 'sasu':
                if ($currentDay == 'sa'
                    || $currentDay == 'su'
                ) {
                    $openDay = true;
                }
                break;

            case 'everyday':
                $openDay = true;
                break;

            default:
                if ($currentDay == $timeData['range']) {
                    $openDay = true;
                }
        }

        if ($timeData['open24']) {
            $openTime = true;
        } else {
            if (!$timeData['closed24']) {
                // @todo Aktuelle Zeit in Location anhand von Geo-Daten holen
                if ($currentTime >= (int)$timeData['from']
                    && $currentTime <= (int)$timeData['to']
                ) {
                    $openTime = true;
                }
            }
        }

        return $openDay && $openTime;
    }

    /**
     * Log watcher activities
     *
     * @param string $call
     * @param array $params
     * @param array $result
     *
     * @return string
     */
    public static function watcherLog($call, $params, $result)
    {
        $L    = QUI::getLocale();
        $lg   = 'quiqqer/locations';
        $prfx = 'watcher.log.';
        $str  = '';

        switch ($call) {
            case 'package_quiqqer_locations_ajax_locations_update':
                $str = $L->get($lg, $prfx.'location.update', [
                    'locationId' => $params['locationId']
                ]);
                break;

            case 'package_quiqqer_locations_ajax_locations_activate':
                $str = $L->get($lg, $prfx.'location.activate', [
                    'locationId' => $params['locationId']
                ]);
                break;

            case 'package_quiqqer_locations_ajax_locations_deactivate':
                $str = $L->get($lg, $prfx.'location.deactivate', [
                    'locationId' => $params['locationId']
                ]);
                break;

            case 'package_quiqqer_locations_ajax_locations_delete':
                $str = $L->get($lg, $prfx.'location.delete');
                break;

            case 'package_quiqqer_locations_ajax_locations_create':
                $str = $L->get($lg, $prfx.'location.create', [
                    'location'   => $params['locationName'],
                    'locationId' => $result['result']
                ]);
                break;
        }

        return $str;
    }

    /**
     * Convert the recurrence setting of a SpecialOpeningHour to view text
     *
     * @param array $recurrence
     * @return array
     */
    public static function convertSpecialOpeningRecurrence(array $recurrence)
    {
        $Locale  = QUI\Locations\Handler\Locations::getLocale();
        $options = $recurrence['options'];
        $text    = '';

        switch ($recurrence['type']) {
            case 'daterange':
                $From = \date_create($options['from']);
                $from = '';

                if ($From) {
                    $from = $Locale->formatDate($From->getTimestamp());
                }

                $To = \date_create($options['to']);
                $to = '';

                if ($To) {
                    $to = $Locale->formatDate($To->getTimestamp());
                }

                $text = $Locale->get(
                    'quiqqer/locations',
                    'specialOpeningHours.view.text.daterange',
                    [
                        'from' => $from,
                        'to'   => $to
                    ]
                );
                break;

            case 'yearly':
                $from = '';

                if (!empty($options['from'])) {
                    $parts = \explode('-', $options['from']);
                    $day   = $parts[1];
                    $month = $parts[0];

                    $from = $day.'.'.$month.'.';
                }

                $to = '';

                if (!empty($options['to'])) {
                    $parts = \explode('-', $options['to']);
                    $day   = $parts[1];
                    $month = $parts[0];

                    $to = $day.'.'.$month.'.';
                }

                $text = $Locale->get(
                    'quiqqer/locations',
                    'specialOpeningHours.view.text.yearly',
                    [
                        'from' => $from,
                        'to'   => $to
                    ]
                );
                break;

            case 'specialday':
                switch ($options['daynum']) {
                    case 'every':
                        $prefix = $Locale->get(
                            'quiqqer/locations',
                            'specialOpeningHours.view.text.specialday.prefix.every'
                        );
                        break;

                    default:
                        $prefix = $Locale->get(
                            'quiqqer/locations',
                            'specialOpeningHours.view.text.specialday.prefix.every_no',
                            [
                                'no' => $options['daynum']
                            ]
                        );
                }

                switch ($options['month']) {
                    case 'month':
                        $suffix = $Locale->get(
                            'quiqqer/locations',
                            'specialOpeningHours.view.text.specialday.suffix.month'
                        );
                        break;

                    default:
                        $suffix = $Locale->get(
                            'quiqqer/locations',
                            'controls.attributes.recurrenceselect.specialday.month.'.$options['month']
                        );
                }

                $text = $prefix.' '.$suffix;
                break;
        }

        return $text;
    }

    /**
     * Convert the configuration of an opening entry to readable text
     *
     * @param array $data
     * @return string[] - Containing text for day and time
     */
    public static function convertOpeningDataToText($data)
    {
        $Locale = QUI\Locations\Handler\Locations::getLocale();
        $text   = [
            'day'  => $Locale->get('quiqqer/locations', 'attribute.OpeningHours.range.'.$data['range']),
            'time' => ''
        ];

        if ($data['closed24']) {
            $text['time'] = $Locale->get('quiqqer/locations', 'attribute.OpeningHours.closed24');
        } elseif ($data['open24']) {
            $text['time'] = $Locale->get('quiqqer/locations', 'attribute.OpeningHours.open24');
        } else {
            $from = '23:59';
            $to   = '00:00';

            if (!empty($data['from'])) {
                $from = mb_substr($data['from'], 0, 2).':'.mb_substr($data['from'], 2);
            }

            if (!empty($data['to'])) {
                $to = mb_substr($data['to'], 0, 2).':'.mb_substr($data['to'], 2);
            }

            $timestampFrom = strtotime($from);
            $timestampTo   = strtotime($to);

            switch ($Locale->getCurrent()) {
                case 'de':
                    $text['time'] = date("H:i", $timestampFrom).' - '.date("H:i", $timestampTo);
                    break;

                default:
                    $text['time'] = date("h:i a", $timestampFrom).' - '.date("h:i a", $timestampTo);
            }
        }

        return $text;
    }
}
